@@ -7,7 +7,11 @@
 /** \file
  * \ingroup bke
  */
-#include "IMB_imbuf_types.h"
+
+#include "DNA_object_enums.h"
+#include "DNA_view3d_enums.h"
+
+#include "IMB_imbuf_types.hh"
 
 #ifdef __cplusplus
 extern "C" {
@@ -29,6 +33,7 @@ typedef struct ImBuf *(*SequencerDrawView)(struct Depsgraph *depsgraph,
                                            int alpha_mode,
                                            const char *viewname,
                                            struct GPUOffScreen *ofs,
+                                           struct GPUViewport *viewport,
                                            char err_out[256]);
 extern SequencerDrawView sequencer_view3d_fn;
 
