@@ -4,31 +4,23 @@
 
 #include "BLI_bounds.hh"
 #include "BLI_map.hh"
+#include "BLI_memory_counter.hh"
 #include "BLI_task.hh"
 
-#include "BLT_translation.h"
-
-#include "BKE_attribute.h"
+#include "BKE_attribute.hh"
 #include "BKE_curves.hh"
 #include "BKE_geometry_set.hh"
 #include "BKE_geometry_set_instances.hh"
 #include "BKE_grease_pencil.hh"
 #include "BKE_instances.hh"
-#include "BKE_lib_id.h"
 #include "BKE_mesh.hh"
-#include "BKE_mesh_wrapper.hh"
-#include "BKE_modifier.h"
-#include "BKE_pointcloud.h"
-#include "BKE_volume.h"
+#include "BKE_modifier.hh"
+#include "BKE_object_types.hh"
+#include "BKE_volume.hh"
 
-#include "DNA_collection_types.h"
 #include "DNA_object_types.h"
 #include "DNA_pointcloud_types.h"
 
-#include "BLI_rand.hh"
-
-#include "MEM_guardedalloc.h"
-
 namespace blender::bke {
 
 /* -------------------------------------------------------------------- */
@@ -41,25 +33,25 @@ GeometryComponentPtr GeometryComponent::create(Type component_type)
 {
   switch (component_type) {
     case Type::Mesh:
-      return new MeshComponent();
+      return GeometryComponentPtr(new MeshComponent());
     case Type::PointCloud:
-      return new PointCloudComponent();
+      return GeometryComponentPtr(new PointCloudComponent());
     case Type::Instance:
-      return new InstancesComponent();
+      return GeometryComponentPtr(new InstancesComponent());
     case Type::Volume:
-      return new VolumeComponent();
+      return GeometryComponentPtr(new VolumeComponent());
     case Type::Curve:
-      return new CurveComponent();
+      return GeometryComponentPtr(new CurveComponent());
     case Type::Edit:
-      return new GeometryComponentEditData();
+      return GeometryComponentPtr(new GeometryComponentEditData());
     case Type::GreasePencil:
-      return new GreasePencilComponent();
+      return GeometryComponentPtr(new GreasePencilComponent());
   }
   BLI_assert_unreachable();
   return {};
 }
 
-int GeometryComponent::attribute_domain_size(const eAttrDomain domain) const
+int GeometryComponent::attribute_domain_size(const AttrDomain domain) const
 {
   if (this->is_empty()) {
     return 0;
@@ -80,6 +72,8 @@ std::optional<MutableAttributeAccessor> GeometryComponent::attributes_for_write(
   return std::nullopt;
 }
 
+void GeometryComponent::count_memory(MemoryCounter & /*memory*/) const {}
+
 GeometryComponent::Type GeometryComponent::type() const
 {
   return type_;
@@ -119,17 +113,17 @@ GeometryComponent &GeometrySet::get_component_for_write(GeometryComponent::Type
   if (!component_ptr) {
     /* If the component did not exist before, create a new one. */
     component_ptr = GeometryComponent::create(component_type);
-    return *component_ptr;
   }
-  if (component_ptr->is_mutable()) {
+  else if (component_ptr->is_mutable()) {
     /* If the referenced component is already mutable, return it directly. */
     component_ptr->tag_ensured_mutable();
-    return *component_ptr;
   }
-  /* If the referenced component is shared, make a copy. The copy is not shared and is
-   * therefore mutable. */
-  component_ptr = component_ptr->copy();
-  return *component_ptr;
+  else {
+    /* If the referenced component is shared, make a copy. The copy is not shared and is
+     * therefore mutable. */
+    component_ptr = component_ptr->copy();
+  }
+  return const_cast<GeometryComponent &>(*component_ptr);
 }
 
 GeometryComponent *GeometrySet::get_component_ptr(GeometryComponent::Type type)
@@ -184,7 +178,8 @@ void GeometrySet::add(const GeometryComponent &component)
 {
   BLI_assert(!components_[size_t(component.type())]);
   component.add_user();
-  components_[size_t(component.type())] = const_cast<GeometryComponent *>(&component);
+  components_[size_t(component.type())] = GeometryComponentPtr(
+      const_cast<GeometryComponent *>(&component));
 }
 
 Vector<const GeometryComponent *> GeometrySet::get_components() const
@@ -205,24 +200,16 @@ std::optional<Bounds<float3>> GeometrySet::compute_boundbox_without_instances()
     bounds = bounds::merge(bounds, pointcloud->bounds_min_max());
   }
   if (const Mesh *mesh = this->get_mesh()) {
-    Bounds<float3> mesh_bounds{float3(std::numeric_limits<float>::max()),
-                               float3(std::numeric_limits<float>::lowest())};
-    if (BKE_mesh_wrapper_minmax(mesh, mesh_bounds.min, mesh_bounds.max)) {
-      bounds = bounds::merge(bounds, {mesh_bounds});
-    }
+    bounds = bounds::merge(bounds, mesh->bounds_min_max());
   }
   if (const Volume *volume = this->get_volume()) {
-    Bounds<float3> volume_bounds{float3(std::numeric_limits<float>::max()),
-                                 float3(std::numeric_limits<float>::lowest())};
-    if (BKE_volume_min_max(volume, volume_bounds.min, volume_bounds.max)) {
-      bounds = bounds::merge(bounds, {volume_bounds});
-    }
+    bounds = bounds::merge(bounds, BKE_volume_min_max(volume));
   }
   if (const Curves *curves_id = this->get_curves()) {
     bounds = bounds::merge(bounds, curves_id->geometry.wrap().bounds_min_max());
   }
   if (const GreasePencil *grease_pencil = this->get_grease_pencil()) {
-    bounds = bounds::merge(bounds, grease_pencil->bounds_min_max());
+    bounds = bounds::merge(bounds, grease_pencil->bounds_min_max_eval());
   }
   return bounds;
 }
@@ -231,15 +218,18 @@ std::ostream &operator<<(std::ostream &stream, const GeometrySet &geometry_set)
 {
   Vector<std::string> parts;
   if (const Mesh *mesh = geometry_set.get_mesh()) {
-    parts.append(std::to_string(mesh->totvert) + " verts");
-    parts.append(std::to_string(mesh->totedge) + " edges");
+    parts.append(std::to_string(mesh->verts_num) + " verts");
+    parts.append(std::to_string(mesh->edges_num) + " edges");
     parts.append(std::to_string(mesh->faces_num) + " faces");
-    parts.append(std::to_string(mesh->totloop) + " corners");
+    parts.append(std::to_string(mesh->corners_num) + " corners");
   }
   if (const Curves *curves = geometry_set.get_curves()) {
     parts.append(std::to_string(curves->geometry.point_num) + " control points");
     parts.append(std::to_string(curves->geometry.curve_num) + " curves");
   }
+  if (const GreasePencil *grease_pencil = geometry_set.get_grease_pencil()) {
+    parts.append(std::to_string(grease_pencil->layers().size()) + " Grease Pencil layers");
+  }
   if (const PointCloud *point_cloud = geometry_set.get_pointcloud()) {
     parts.append(std::to_string(point_cloud->totpoint) + " points");
   }
@@ -347,6 +337,18 @@ const CurvesEditHints *GeometrySet::get_curve_edit_hints() const
   return (component == nullptr) ? nullptr : component->curves_edit_hints_.get();
 }
 
+const GreasePencilEditHints *GeometrySet::get_grease_pencil_edit_hints() const
+{
+  const GeometryComponentEditData *component = this->get_component<GeometryComponentEditData>();
+  return (component == nullptr) ? nullptr : component->grease_pencil_edit_hints_.get();
+}
+
+const GizmoEditHints *GeometrySet::get_gizmo_edit_hints() const
+{
+  const GeometryComponentEditData *component = this->get_component<GeometryComponentEditData>();
+  return (component == nullptr) ? nullptr : component->gizmo_edit_hints_.get();
+}
+
 const GreasePencil *GeometrySet::get_grease_pencil() const
 {
   const GreasePencilComponent *component = this->get_component<GreasePencilComponent>();
@@ -570,12 +572,43 @@ CurvesEditHints *GeometrySet::get_curve_edit_hints_for_write()
   return component.curves_edit_hints_.get();
 }
 
+GreasePencilEditHints *GeometrySet::get_grease_pencil_edit_hints_for_write()
+{
+  if (!this->has<GeometryComponentEditData>()) {
+    return nullptr;
+  }
+  GeometryComponentEditData &component =
+      this->get_component_for_write<GeometryComponentEditData>();
+  return component.grease_pencil_edit_hints_.get();
+}
+
+GizmoEditHints *GeometrySet::get_gizmo_edit_hints_for_write()
+{
+  if (!this->has<GeometryComponentEditData>()) {
+    return nullptr;
+  }
+  GeometryComponentEditData &component =
+      this->get_component_for_write<GeometryComponentEditData>();
+  return component.gizmo_edit_hints_.get();
+}
+
 GreasePencil *GeometrySet::get_grease_pencil_for_write()
 {
   GreasePencilComponent *component = this->get_component_ptr<GreasePencilComponent>();
   return component == nullptr ? nullptr : component->get_for_write();
 }
 
+void GeometrySet::count_memory(MemoryCounter &memory) const
+{
+  for (const GeometryComponentPtr &component : components_) {
+    if (component) {
+      memory.add_shared(component.get(), [&](MemoryCounter &shared_memory) {
+        component->count_memory(shared_memory);
+      });
+    }
+  }
+}
+
 void GeometrySet::attribute_foreach(const Span<GeometryComponent::Type> component_types,
                                     const bool include_instances,
                                     const AttributeForeachCallback callback) const
@@ -587,11 +620,9 @@ void GeometrySet::attribute_foreach(const Span<GeometryComponent::Type> componen
     const GeometryComponent &component = *this->get_component(component_type);
     const std::optional<AttributeAccessor> attributes = component.attributes();
     if (attributes.has_value()) {
-      attributes->for_all(
-          [&](const AttributeIDRef &attribute_id, const AttributeMetaData &meta_data) {
-            callback(attribute_id, meta_data, component);
-            return true;
-          });
+      attributes->foreach_attribute([&](const AttributeIter &iter) {
+        callback(iter.name, {iter.domain, iter.data_type}, component);
+      });
     }
   }
   if (include_instances && this->has_instances()) {
@@ -602,24 +633,57 @@ void GeometrySet::attribute_foreach(const Span<GeometryComponent::Type> componen
   }
 }
 
+bool attribute_is_builtin_on_component_type(const GeometryComponent::Type type,
+                                            const StringRef name)
+{
+  switch (type) {
+    case GeometryComponent::Type::Mesh: {
+      static auto component = GeometryComponent::create(type);
+      return component->attributes()->is_builtin(name);
+    }
+    case GeometryComponent::Type::PointCloud: {
+      static auto component = GeometryComponent::create(type);
+      return component->attributes()->is_builtin(name);
+    }
+    case GeometryComponent::Type::Instance: {
+      static auto component = GeometryComponent::create(type);
+      return component->attributes()->is_builtin(name);
+    }
+    case GeometryComponent::Type::Curve: {
+      static auto component = GeometryComponent::create(type);
+      return component->attributes()->is_builtin(name);
+    }
+    case GeometryComponent::Type::GreasePencil: {
+      static auto grease_pencil_component = GeometryComponent::create(
+          GeometryComponent::Type::GreasePencil);
+      static auto curves_component = GeometryComponent::create(GeometryComponent::Type::Curve);
+      return grease_pencil_component->attributes()->is_builtin(name) ||
+             curves_component->attributes()->is_builtin(name);
+    }
+    case GeometryComponent::Type::Volume:
+    case GeometryComponent::Type::Edit: {
+      return false;
+    }
+  }
+  BLI_assert_unreachable();
+  return false;
+}
+
 void GeometrySet::gather_attributes_for_propagation(
     const Span<GeometryComponent::Type> component_types,
     const GeometryComponent::Type dst_component_type,
     bool include_instances,
-    const AnonymousAttributePropagationInfo &propagation_info,
-    Map<AttributeIDRef, AttributeKind> &r_attributes) const
+    const AttributeFilter &attribute_filter,
+    Map<StringRef, AttributeDomainAndType> &r_attributes) const
 {
-  /* Only needed right now to check if an attribute is built-in on this component type.
-   * TODO: Get rid of the dummy component. */
-  const GeometryComponentPtr dummy_component = GeometryComponent::create(dst_component_type);
   this->attribute_foreach(
       component_types,
       include_instances,
-      [&](const AttributeIDRef &attribute_id,
+      [&](const StringRef attribute_id,
           const AttributeMetaData &meta_data,
           const GeometryComponent &component) {
         if (component.attributes()->is_builtin(attribute_id)) {
-          if (!dummy_component->attributes()->is_builtin(attribute_id)) {
+          if (!attribute_is_builtin_on_component_type(dst_component_type, attribute_id)) {
             /* Don't propagate built-in attributes that are not built-in on the destination
              * component. */
             return;
@@ -629,22 +693,21 @@ void GeometrySet::gather_attributes_for_propagation(
           /* Propagating string attributes is not supported yet. */
           return;
         }
-        if (attribute_id.is_anonymous() &&
-            !propagation_info.propagate(attribute_id.anonymous_id())) {
+        if (attribute_filter.allow_skip(attribute_id)) {
           return;
         }
 
-        eAttrDomain domain = meta_data.domain;
+        AttrDomain domain = meta_data.domain;
         if (dst_component_type != GeometryComponent::Type::Instance &&
-            domain == ATTR_DOMAIN_INSTANCE) {
-          domain = ATTR_DOMAIN_POINT;
+            domain == AttrDomain::Instance) {
+          domain = AttrDomain::Point;
         }
 
-        auto add_info = [&](AttributeKind *attribute_kind) {
+        auto add_info = [&](AttributeDomainAndType *attribute_kind) {
           attribute_kind->domain = domain;
           attribute_kind->data_type = meta_data.data_type;
         };
-        auto modify_info = [&](AttributeKind *attribute_kind) {
+        auto modify_info = [&](AttributeDomainAndType *attribute_kind) {
           attribute_kind->domain = bke::attribute_domain_highest_priority(
               {attribute_kind->domain, domain});
           attribute_kind->data_type = bke::attribute_data_type_highest_complexity(
@@ -723,41 +786,29 @@ void GeometrySet::modify_geometry_sets(ForeachSubGeometryCallback callback)
 
 bool object_has_geometry_set_instances(const Object &object)
 {
-  const GeometrySet *geometry_set = object.runtime.geometry_set_eval;
+  const GeometrySet *geometry_set = object.runtime->geometry_set_eval;
   if (geometry_set == nullptr) {
     return false;
   }
-  for (const GeometryComponent *component : geometry_set->get_components()) {
-    if (component->is_empty()) {
-      continue;
-    }
-    const GeometryComponent::Type type = component->type();
-    bool is_instance = false;
-    switch (type) {
-      case GeometryComponent::Type::Mesh:
-        is_instance = object.type != OB_MESH;
-        break;
-      case GeometryComponent::Type::PointCloud:
-        is_instance = object.type != OB_POINTCLOUD;
-        break;
-      case GeometryComponent::Type::Instance:
-        is_instance = true;
-        break;
-      case GeometryComponent::Type::Volume:
-        is_instance = object.type != OB_VOLUME;
-        break;
-      case GeometryComponent::Type::Curve:
-        is_instance = !ELEM(object.type, OB_CURVES_LEGACY, OB_FONT);
-        break;
-      case GeometryComponent::Type::Edit:
-        break;
-      case GeometryComponent::Type::GreasePencil:
-        is_instance = object.type != OB_GREASE_PENCIL;
-        break;
-    }
-    if (is_instance) {
-      return true;
-    }
+  if (geometry_set->has_component<InstancesComponent>()) {
+    return true;
+  }
+  if (object.type != OB_MESH && geometry_set->has_component<MeshComponent>()) {
+    return true;
+  }
+  if (object.type != OB_POINTCLOUD && geometry_set->has_component<PointCloudComponent>()) {
+    return true;
+  }
+  if (object.type != OB_VOLUME && geometry_set->has_component<VolumeComponent>()) {
+    return true;
+  }
+  if (!ELEM(object.type, OB_CURVES_LEGACY, OB_FONT) &&
+      geometry_set->has_component<CurveComponent>())
+  {
+    return true;
+  }
+  if (object.type != OB_GREASE_PENCIL && geometry_set->has_component<GreasePencilComponent>()) {
+    return true;
   }
   return false;
 }
