@@ -6,47 +6,46 @@
  * \ingroup render
  */
 
-#include <cmath>
+#include <condition_variable>
 #include <cstddef>
 #include <cstring>
+#include <mutex>
 
 #include "MEM_guardedalloc.h"
 
 #include "BLI_bitmap.h"
-#include "BLI_blenlib.h"
+#include "BLI_fileops.h"
 #include "BLI_math_color_blend.h"
+#include "BLI_string.h"
 #include "BLI_task.h"
 #include "BLI_task.hh"
-#include "BLI_threads.h"
 #include "BLI_utildefines.h"
-#include "DNA_camera_types.h"
+#include "BLI_vector.hh"
 
-#include "DNA_action_types.h"
 #include "DNA_anim_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_gpencil_legacy_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 
-#include "BKE_anim_data.h"
+#include "BKE_anim_data.hh"
 #include "BKE_camera.h"
-#include "BKE_context.h"
-#include "BKE_customdata.h"
-#include "BKE_fcurve.h"
-#include "BKE_global.h"
-#include "BKE_image.h"
-#include "BKE_image_format.h"
-#include "BKE_image_save.h"
-#include "BKE_lib_query.h"
-#include "BKE_main.h"
-#include "BKE_report.h"
-#include "BKE_scene.h"
-#include "BKE_writeavi.h"
+#include "BKE_context.hh"
+#include "BKE_customdata.hh"
+#include "BKE_fcurve.hh"
+#include "BKE_global.hh"
+#include "BKE_image.hh"
+#include "BKE_image_format.hh"
+#include "BKE_image_save.hh"
+#include "BKE_lib_query.hh"
+#include "BKE_main.hh"
+#include "BKE_report.hh"
+#include "BKE_scene.hh"
 
 #include "DEG_depsgraph.hh"
 #include "DEG_depsgraph_query.hh"
 
-#include "DRW_engine.h"
+#include "DRW_engine.hh"
 
 #include "WM_api.hh"
 #include "WM_types.hh"
@@ -56,99 +55,95 @@
 #include "ED_view3d.hh"
 #include "ED_view3d_offscreen.hh"
 
-#include "IMB_colormanagement.h"
-#include "IMB_imbuf.h"
-#include "IMB_imbuf_types.h"
+#include "IMB_imbuf.hh"
+#include "IMB_imbuf_types.hh"
+
+#include "MOV_write.hh"
 
 #include "RE_pipeline.h"
 
-#include "BLT_translation.h"
+#include "BLT_translation.hh"
 
 #include "RNA_access.hh"
 #include "RNA_define.hh"
 
-#include "SEQ_render.h"
+#include "SEQ_render.hh"
 
-#include "GPU_framebuffer.h"
-#include "GPU_matrix.h"
+#include "ANIM_action_legacy.hh"
+
+#include "GPU_context.hh"
+#include "GPU_framebuffer.hh"
+#include "GPU_matrix.hh"
+#include "GPU_state.hh"
+#include "GPU_viewport.hh"
 
 #include "render_intern.hh"
 
-/* Define this to get timing information. */
-// #define DEBUG_TIME
-
-#ifdef DEBUG_TIME
-#  include "PIL_time.h"
-#endif
-
 /* TODO(sergey): Find better approximation of the scheduled frames.
  * For really high-resolution renders it might fail still. */
 #define MAX_SCHEDULED_FRAMES 8
 
-struct OGLRender {
-  Main *bmain;
-  Render *re;
-  Scene *scene;
-  WorkSpace *workspace;
-  ViewLayer *view_layer;
-  Depsgraph *depsgraph;
+struct OGLRender : public RenderJobBase {
+  Main *bmain = nullptr;
+  Render *re = nullptr;
+  WorkSpace *workspace = nullptr;
+  ViewLayer *view_layer = nullptr;
+  Depsgraph *depsgraph = nullptr;
 
-  View3D *v3d;
-  RegionView3D *rv3d;
-  ARegion *region;
+  View3D *v3d = nullptr;
+  RegionView3D *rv3d = nullptr;
+  ARegion *region = nullptr;
 
-  ScrArea *prevsa;
-  ARegion *prevar;
+  int views_len = 0; /* multi-view views */
 
-  int views_len; /* multi-view views */
-
-  bool is_sequencer;
-  SpaceSeq *sseq;
+  bool is_sequencer = false;
+  SpaceSeq *sseq = nullptr;
   struct {
-    ImBuf **ibufs_arr;
+    ImBuf **ibufs_arr = nullptr;
   } seq_data;
 
-  Image *ima;
-  ImageUser iuser;
+  Image *ima = nullptr;
+  ImageUser iuser = {};
 
-  GPUOffScreen *ofs;
-  int sizex, sizey;
-  int write_still;
+  GPUOffScreen *ofs = nullptr;
+  int sizex = 0;
+  int sizey = 0;
+  int write_still = false;
 
-  ReportList *reports;
-  bMovieHandle *mh;
-  int cfrao, nfra;
+  GPUViewport *viewport = nullptr;
 
-  int totvideos;
+  std::mutex reports_mutex;
+  ReportList *reports = nullptr;
+
+  int cfrao = 0;
+  int nfra = 0;
+
+  int totvideos = 0;
 
   /* For only rendering frames that have a key in animation data. */
-  BLI_bitmap *render_frames;
+  BLI_bitmap *render_frames = nullptr;
 
   /* quick lookup */
-  int view_id;
+  int view_id = 0;
 
   /* wm vars for timer and progress cursor */
-  wmWindowManager *wm;
-  wmWindow *win;
+  wmWindowManager *wm = nullptr;
+  wmWindow *win = nullptr;
 
-  /** Use to check if running modal or not (invoked or executed). */
-  wmTimer *timer;
+  blender::Vector<MovieWriter *> movie_writers;
 
-  void **movie_ctx_arr;
+  TaskPool *task_pool = nullptr;
+  bool pool_ok = true;
+  bool is_animation = false;
 
-  TaskPool *task_pool;
-  bool pool_ok;
-  bool is_animation;
+  eImageFormatDepth color_depth = R_IMF_CHAN_DEPTH_32;
+  uint num_scheduled_frames = 0;
+  std::mutex task_mutex;
+  std::condition_variable task_condition;
 
-  eImageFormatDepth color_depth;
-  SpinLock reports_lock;
-  uint num_scheduled_frames;
-  ThreadMutex task_mutex;
-  ThreadCondition task_condition;
+  wmJob *wm_job = nullptr;
 
-#ifdef DEBUG_TIME
-  double time_start;
-#endif
+  bool ended = false;
 };
 
 static bool screen_opengl_is_multiview(OGLRender *oglrender)
@@ -256,7 +251,7 @@ static void screen_opengl_views_setup(OGLRender *oglrender)
   RE_ReleaseResult(oglrender->re);
 }
 
-static void screen_opengl_render_doit(const bContext *C, OGLRender *oglrender, RenderResult *rr)
+static void screen_opengl_render_doit(OGLRender *oglrender, RenderResult *rr)
 {
   Scene *scene = oglrender->scene;
   Object *camera = nullptr;
@@ -326,7 +321,7 @@ static void screen_opengl_render_doit(const bContext *C, OGLRender *oglrender, R
   }
   else {
     /* shouldn't suddenly give errors mid-render but possible */
-    Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
+    Depsgraph *depsgraph = oglrender->depsgraph;
     char err_out[256] = "unknown";
     ImBuf *ibuf_view;
     bool draw_sky = (scene->r.alphamode == R_ADDSKY);
@@ -334,6 +329,8 @@ static void screen_opengl_render_doit(const bContext *C, OGLRender *oglrender, R
     const char *viewname = RE_GetActiveRenderView(oglrender->re);
     View3D *v3d = oglrender->v3d;
 
+    BKE_scene_graph_evaluated_ensure(depsgraph, oglrender->bmain);
+
     if (v3d != nullptr) {
       ARegion *region = oglrender->region;
       ibuf_view = ED_view3d_draw_offscreen_imbuf(depsgraph,
@@ -348,6 +345,7 @@ static void screen_opengl_render_doit(const bContext *C, OGLRender *oglrender, R
                                                  viewname,
                                                  true,
                                                  oglrender->ofs,
+                                                 oglrender->viewport,
                                                  err_out);
 
       /* for stamp only */
@@ -368,6 +366,7 @@ static void screen_opengl_render_doit(const bContext *C, OGLRender *oglrender, R
                                                         alpha_mode,
                                                         viewname,
                                                         oglrender->ofs,
+                                                        oglrender->viewport,
                                                         err_out);
       camera = scene->camera;
     }
@@ -390,11 +389,20 @@ static void screen_opengl_render_doit(const bContext *C, OGLRender *oglrender, R
       else {
         rect = ibuf_result->byte_buffer.data;
       }
-      BKE_image_stamp_buf(scene, camera, nullptr, rect, rectf, rr->rectx, rr->recty, 4);
+      BKE_image_stamp_buf(scene, camera, nullptr, rect, rectf, rr->rectx, rr->recty);
     }
     RE_render_result_rect_from_ibuf(rr, ibuf_result, oglrender->view_id);
     IMB_freeImBuf(ibuf_result);
   }
+
+  /* Perform render step between renders to allow
+   * flushing of freed GPUBackend resources. */
+  DRW_gpu_context_enable();
+  if (GPU_backend_get_type() == GPU_BACKEND_METAL) {
+    GPU_flush();
+  }
+  GPU_render_step(true);
+  DRW_gpu_context_disable();
 }
 
 static void screen_opengl_render_write(OGLRender *oglrender)
@@ -442,7 +450,7 @@ static void UNUSED_FUNCTION(addAlphaOverFloat)(float dest[4], const float source
   dest[3] = (mul * dest[3]) + source[3];
 }
 
-static void screen_opengl_render_apply(const bContext *C, OGLRender *oglrender)
+static void screen_opengl_render_apply(OGLRender *oglrender)
 {
   RenderResult *rr;
   RenderView *rv;
@@ -469,6 +477,7 @@ static void screen_opengl_render_apply(const bContext *C, OGLRender *oglrender)
     for (view_id = 0; view_id < oglrender->views_len; view_id++) {
       context.view_id = view_id;
       context.gpu_offscreen = oglrender->ofs;
+      context.gpu_viewport = oglrender->viewport;
       oglrender->seq_data.ibufs_arr[view_id] = SEQ_render_give_ibuf(
           &context, scene->r.cfra, chanshown);
     }
@@ -481,7 +490,7 @@ static void screen_opengl_render_apply(const bContext *C, OGLRender *oglrender)
     RE_SetActiveRenderView(oglrender->re, rv->name);
     oglrender->view_id = view_id;
     /* render composite */
-    screen_opengl_render_doit(C, oglrender, rr);
+    screen_opengl_render_doit(oglrender, rr);
   }
 
   RE_ReleaseResult(oglrender->re);
@@ -508,7 +517,7 @@ static void gather_frames_to_render_for_adt(const OGLRender *oglrender, const An
   int frame_start = PSFRA;
   int frame_end = PEFRA;
 
-  LISTBASE_FOREACH (FCurve *, fcu, &adt->action->curves) {
+  for (const FCurve *fcu : blender::animrig::legacy::fcurves_for_assigned_action(adt)) {
     if (fcu->driver != nullptr || fcu->fpt != nullptr) {
       /* Drivers have values for any point in time, so to get "the keyed frames" they are
        * useless. Same for baked FCurves, they also have keys for every frame, which is not
@@ -565,7 +574,7 @@ static int gather_frames_to_render_for_id(LibraryIDLinkCallbackData *cb_data)
   ID *id = *id_p;
 
   ID *self_id = cb_data->self_id;
-  const int cb_flag = cb_data->cb_flag;
+  const LibraryForeachIDCallbackFlag cb_flag = cb_data->cb_flag;
   if (cb_flag == IDWALK_CB_LOOPBACK || id == self_id) {
     /* IDs may end up referencing themselves one way or the other, and those
      * (the self_id ones) have always already been processed. */
@@ -684,8 +693,8 @@ static bool screen_opengl_render_init(bContext *C, wmOperator *op)
   WorkSpace *workspace = CTX_wm_workspace(C);
 
   Scene *scene = CTX_data_scene(C);
-  ScrArea *prevsa = CTX_wm_area(C);
-  ARegion *prevar = CTX_wm_region(C);
+  ScrArea *prev_area = CTX_wm_area(C);
+  ARegion *prev_region = CTX_wm_region(C);
   GPUOffScreen *ofs;
   OGLRender *oglrender;
   int sizex, sizey;
@@ -709,6 +718,12 @@ static bool screen_opengl_render_init(bContext *C, wmOperator *op)
     return false;
   }
 
+  if (!is_animation && is_write_still && BKE_imtype_is_movie(scene->r.im_format.imtype)) {
+    BKE_report(
+        op->reports, RPT_ERROR, "Cannot write a single file with an animation format selected");
+    return false;
+  }
+
   if (is_sequencer) {
     is_view_context = false;
   }
@@ -725,12 +740,6 @@ static bool screen_opengl_render_init(bContext *C, wmOperator *op)
     }
   }
 
-  if (!is_animation && is_write_still && BKE_imtype_is_movie(scene->r.im_format.imtype)) {
-    BKE_report(
-        op->reports, RPT_ERROR, "Cannot write a single file with an animation format selected");
-    return false;
-  }
-
   /* stop all running jobs, except screen one. currently previews frustrate Render */
   WM_jobs_kill_all_except(wm, CTX_wm_screen(C));
 
@@ -749,6 +758,8 @@ static bool screen_opengl_render_init(bContext *C, wmOperator *op)
 
   if (!ofs) {
     BKE_reportf(op->reports, RPT_ERROR, "Failed to create OpenGL off-screen buffer, %s", err_out);
+    CTX_wm_area_set(C, prev_area);
+    CTX_wm_region_set(C, prev_region);
     return false;
   }
 
@@ -759,8 +770,10 @@ static bool screen_opengl_render_init(bContext *C, wmOperator *op)
   oglrender->ofs = ofs;
   oglrender->sizex = sizex;
   oglrender->sizey = sizey;
+  oglrender->viewport = GPU_viewport_create();
   oglrender->bmain = CTX_data_main(C);
   oglrender->scene = scene;
+  oglrender->current_scene = scene;
   oglrender->workspace = workspace;
   oglrender->view_layer = CTX_data_view_layer(C);
   /* NOTE: The depsgraph is not only used to update scene for a new frames, but also to initialize
@@ -782,12 +795,13 @@ static bool screen_opengl_render_init(bContext *C, wmOperator *op)
     oglrender->seq_data.ibufs_arr = ibufs_arr;
   }
 
-  oglrender->prevsa = prevsa;
-  oglrender->prevar = prevar;
-
   if (is_view_context) {
-    /* so quad view renders camera */
-    ED_view3d_context_user_region(C, &oglrender->v3d, &oglrender->region);
+    /* Prefer rendering camera in quad view if possible. */
+    if (!ED_view3d_context_user_region(C, &oglrender->v3d, &oglrender->region)) {
+      /* If not get region activated by ED_view3d_context_activate earlier. */
+      oglrender->v3d = CTX_wm_view3d(C);
+      oglrender->region = CTX_wm_region(C);
+    }
 
     oglrender->rv3d = static_cast<RegionView3D *>(oglrender->region->regiondata);
 
@@ -827,10 +841,6 @@ static bool screen_opengl_render_init(bContext *C, wmOperator *op)
   oglrender->wm = wm;
   oglrender->win = win;
 
-  oglrender->totvideos = 0;
-  oglrender->mh = nullptr;
-  oglrender->movie_ctx_arr = nullptr;
-
   if (is_animation) {
     if (is_render_keyed_only) {
       gather_frames_to_render(C, oglrender);
@@ -842,29 +852,22 @@ static bool screen_opengl_render_init(bContext *C, wmOperator *op)
     else {
       oglrender->task_pool = BLI_task_pool_create(oglrender, TASK_PRIORITY_HIGH);
     }
-    oglrender->pool_ok = true;
-    BLI_spin_init(&oglrender->reports_lock);
   }
-  else {
-    oglrender->task_pool = nullptr;
-  }
-  oglrender->num_scheduled_frames = 0;
-  BLI_mutex_init(&oglrender->task_mutex);
-  BLI_condition_init(&oglrender->task_condition);
 
-#ifdef DEBUG_TIME
-  oglrender->time_start = PIL_check_seconds_timer();
-#endif
+  CTX_wm_area_set(C, prev_area);
+  CTX_wm_region_set(C, prev_region);
 
   return true;
 }
 
-static void screen_opengl_render_end(bContext *C, OGLRender *oglrender)
+static void screen_opengl_render_end(OGLRender *oglrender)
 {
-  Scene *scene = oglrender->scene;
-  int i;
+  /* Ensure we don't call this both from the job and operator callbacks. */
+  if (oglrender->ended) {
+    return;
+  }
 
-  if (oglrender->is_animation) {
+  if (oglrender->task_pool) {
     /* Trickery part for movie output:
      *
      * We MUST write frames in an exact order, so we only let background
@@ -874,83 +877,78 @@ static void screen_opengl_render_end(bContext *C, OGLRender *oglrender)
      * After this loop is done work_and_wait() will have nothing to do,
      * so we don't run into wrong order of frames written to the stream.
      */
-    if (BKE_imtype_is_movie(scene->r.im_format.imtype)) {
-      BLI_mutex_lock(&oglrender->task_mutex);
+    if (BKE_imtype_is_movie(oglrender->scene->r.im_format.imtype)) {
+      std::unique_lock lock(oglrender->task_mutex);
       while (oglrender->num_scheduled_frames > 0) {
-        BLI_condition_wait(&oglrender->task_condition, &oglrender->task_mutex);
+        oglrender->task_condition.wait(lock);
       }
-      BLI_mutex_unlock(&oglrender->task_mutex);
     }
+
     BLI_task_pool_work_and_wait(oglrender->task_pool);
     BLI_task_pool_free(oglrender->task_pool);
-    BLI_spin_end(&oglrender->reports_lock);
+    oglrender->task_pool = nullptr;
   }
-  BLI_mutex_end(&oglrender->task_mutex);
-  BLI_condition_end(&oglrender->task_condition);
-
-#ifdef DEBUG_TIME
-  printf("Total render time: %f\n", PIL_check_seconds_timer() - oglrender->time_start);
-#endif
 
   MEM_SAFE_FREE(oglrender->render_frames);
 
-  if (oglrender->mh) {
-    if (BKE_imtype_is_movie(scene->r.im_format.imtype)) {
-      for (i = 0; i < oglrender->totvideos; i++) {
-        oglrender->mh->end_movie(oglrender->movie_ctx_arr[i]);
-        oglrender->mh->context_free(oglrender->movie_ctx_arr[i]);
+  if (!oglrender->movie_writers.is_empty()) {
+    if (BKE_imtype_is_movie(oglrender->scene->r.im_format.imtype)) {
+      for (MovieWriter *writer : oglrender->movie_writers) {
+        MOV_write_end(writer);
       }
     }
-
-    if (oglrender->movie_ctx_arr) {
-      MEM_freeN(oglrender->movie_ctx_arr);
-    }
+    oglrender->movie_writers.clear_and_shrink();
   }
 
-  if (oglrender->timer) { /* exec will not have a timer */
-    Depsgraph *depsgraph = oglrender->depsgraph;
-    scene->r.cfra = oglrender->cfrao;
-    BKE_scene_graph_update_for_newframe(depsgraph);
+  if (oglrender->ofs || oglrender->viewport) {
+    DRW_gpu_context_enable();
+    GPU_offscreen_free(oglrender->ofs);
+    GPU_viewport_free(oglrender->viewport);
+    DRW_gpu_context_disable();
 
-    WM_event_timer_remove(oglrender->wm, oglrender->win, oglrender->timer);
+    oglrender->ofs = nullptr;
+    oglrender->viewport = nullptr;
   }
 
-  WM_cursor_modal_restore(oglrender->win);
-
-  WM_event_add_notifier(C, NC_SCENE | ND_RENDER_RESULT, oglrender->scene);
-
-  DRW_gpu_context_enable();
-  GPU_offscreen_free(oglrender->ofs);
-  DRW_gpu_context_disable();
-
-  if (oglrender->is_sequencer) {
-    MEM_freeN(oglrender->seq_data.ibufs_arr);
-  }
+  MEM_SAFE_FREE(oglrender->seq_data.ibufs_arr);
 
   memset(&oglrender->scene->customdata_mask_modal,
          0,
          sizeof(oglrender->scene->customdata_mask_modal));
 
-  CTX_wm_area_set(C, oglrender->prevsa);
-  CTX_wm_region_set(C, oglrender->prevar);
+  if (oglrender->wm_job) { /* exec will not have a job */
+    Depsgraph *depsgraph = oglrender->depsgraph;
+    oglrender->scene->r.cfra = oglrender->cfrao;
+    BKE_scene_graph_update_for_newframe(depsgraph);
+  }
+  else if (oglrender->win) {
+    WM_cursor_modal_restore(oglrender->win);
+  }
 
-  MEM_delete(oglrender);
+  WM_main_add_notifier(NC_SCENE | ND_RENDER_RESULT, oglrender->scene);
+  G.is_rendering = false;
+  oglrender->ended = true;
 }
 
 static void screen_opengl_render_cancel(bContext *C, wmOperator *op)
 {
-  screen_opengl_render_end(C, static_cast<OGLRender *>(op->customdata));
+  wmWindowManager *wm = CTX_wm_manager(C);
+  OGLRender *oglrender = static_cast<OGLRender *>(op->customdata);
+
+  if (oglrender->is_animation) {
+    WM_jobs_kill_type(wm, oglrender->scene, WM_JOB_TYPE_RENDER);
+  }
+
+  screen_opengl_render_end(oglrender);
+  MEM_delete(oglrender);
 }
 
 /* share between invoke and exec */
-static bool screen_opengl_render_anim_init(bContext *C, wmOperator *op)
+static bool screen_opengl_render_anim_init(wmOperator *op)
 {
   /* initialize animation */
-  OGLRender *oglrender;
-  Scene *scene;
-
-  oglrender = static_cast<OGLRender *>(op->customdata);
-  scene = oglrender->scene;
+  OGLRender *oglrender = static_cast<OGLRender *>(op->customdata);
+  Scene *scene = oglrender->scene;
   oglrender->totvideos = BKE_scene_multiview_num_videos_get(&scene->r);
 
   oglrender->reports = op->reports;
@@ -961,37 +959,29 @@ static bool screen_opengl_render_anim_init(bContext *C, wmOperator *op)
 
     BKE_scene_multiview_videos_dimensions_get(
         &scene->r, oglrender->sizex, oglrender->sizey, &width, &height);
-    oglrender->mh = BKE_movie_handle_get(scene->r.im_format.imtype);
-
-    if (oglrender->mh == nullptr) {
-      BKE_report(oglrender->reports, RPT_ERROR, "Movie format unsupported");
-      screen_opengl_render_end(C, oglrender);
-      return false;
-    }
-
-    oglrender->movie_ctx_arr = static_cast<void **>(
-        MEM_mallocN(sizeof(void *) * oglrender->totvideos, "Movies"));
+    oglrender->movie_writers.reserve(oglrender->totvideos);
 
     for (i = 0; i < oglrender->totvideos; i++) {
       Scene *scene_eval = DEG_get_evaluated_scene(oglrender->depsgraph);
       const char *suffix = BKE_scene_multiview_view_id_suffix_get(&scene->r, i);
-
-      oglrender->movie_ctx_arr[i] = oglrender->mh->context_create();
-      if (!oglrender->mh->start_movie(oglrender->movie_ctx_arr[i],
-                                      scene_eval,
-                                      &scene->r,
-                                      oglrender->sizex,
-                                      oglrender->sizey,
-                                      oglrender->reports,
-                                      PRVRANGEON != 0,
-                                      suffix))
-      {
-        screen_opengl_render_end(C, oglrender);
+      MovieWriter *writer = MOV_write_begin(scene->r.im_format.imtype,
+                                            scene_eval,
+                                            &scene->r,
+                                            oglrender->sizex,
+                                            oglrender->sizey,
+                                            oglrender->reports,
+                                            PRVRANGEON != 0,
+                                            suffix);
+      if (writer == nullptr) {
+        screen_opengl_render_end(oglrender);
+        MEM_delete(oglrender);
         return false;
       }
+      oglrender->movie_writers.append(writer);
     }
   }
 
+  G.is_rendering = true;
   oglrender->cfrao = scene->r.cfra;
   oglrender->nfra = PSFRA;
   scene->r.cfra = PSFRA;
@@ -1015,10 +1005,9 @@ static void write_result(TaskPool *__restrict pool, WriteTaskData *task_data)
   /* Don't attempt to write if we've got an error. */
   if (!oglrender->pool_ok) {
     RE_FreeRenderResult(rr);
-    BLI_mutex_lock(&oglrender->task_mutex);
+    std::scoped_lock lock(oglrender->task_mutex);
     oglrender->num_scheduled_frames--;
-    BLI_condition_notify_all(&oglrender->task_condition);
-    BLI_mutex_unlock(&oglrender->task_mutex);
+    oglrender->task_condition.notify_all();
     return;
   }
   /* Construct local thread0safe copy of reports structure which we can
@@ -1037,8 +1026,7 @@ static void write_result(TaskPool *__restrict pool, WriteTaskData *task_data)
                                   rr,
                                   scene,
                                   &scene->r,
-                                  oglrender->mh,
-                                  oglrender->movie_ctx_arr,
+                                  oglrender->movie_writers.data(),
                                   oglrender->totvideos,
                                   PRVRANGEON != 0);
   }
@@ -1063,22 +1051,25 @@ static void write_result(TaskPool *__restrict pool, WriteTaskData *task_data)
     }
   }
   if (reports.list.first != nullptr) {
-    BLI_spin_lock(&oglrender->reports_lock);
+    /* TODO: Should rather use new #BKE_reports_move_to_reports ? */
+    std::unique_lock lock(oglrender->reports_mutex);
     for (Report *report = static_cast<Report *>(reports.list.first); report != nullptr;
          report = report->next)
     {
       BKE_report(oglrender->reports, static_cast<eReportType>(report->type), report->message);
     }
-    BLI_spin_unlock(&oglrender->reports_lock);
   }
+  BKE_reports_free(&reports);
   if (!ok) {
     oglrender->pool_ok = false;
   }
   RE_FreeRenderResult(rr);
-  BLI_mutex_lock(&oglrender->task_mutex);
-  oglrender->num_scheduled_frames--;
-  BLI_condition_notify_all(&oglrender->task_condition);
-  BLI_mutex_unlock(&oglrender->task_mutex);
+
+  {
+    std::unique_lock lock(oglrender->task_mutex);
+    oglrender->num_scheduled_frames--;
+    oglrender->task_condition.notify_all();
+  }
 }
 
 static void write_result_func(TaskPool *__restrict pool, void *task_data_v)
@@ -1097,22 +1088,22 @@ static bool schedule_write_result(OGLRender *oglrender, RenderResult *rr)
     return false;
   }
   Scene *scene = oglrender->scene;
-  WriteTaskData *task_data = MEM_new<WriteTaskData>("write task data");
+  WriteTaskData *task_data = MEM_cnew<WriteTaskData>("write task data");
   task_data->rr = rr;
   memcpy(&task_data->tmp_scene, scene, sizeof(task_data->tmp_scene));
-  BLI_mutex_lock(&oglrender->task_mutex);
-  oglrender->num_scheduled_frames++;
-  if (oglrender->num_scheduled_frames > MAX_SCHEDULED_FRAMES) {
-    BLI_condition_wait(&oglrender->task_condition, &oglrender->task_mutex);
+  {
+    std::unique_lock lock(oglrender->task_mutex);
+    oglrender->num_scheduled_frames++;
+    if (oglrender->num_scheduled_frames > MAX_SCHEDULED_FRAMES) {
+      oglrender->task_condition.wait(lock);
+    }
   }
-  BLI_mutex_unlock(&oglrender->task_mutex);
   BLI_task_pool_push(oglrender->task_pool, write_result_func, task_data, true, nullptr);
   return true;
 }
 
-static bool screen_opengl_render_anim_step(bContext *C, wmOperator *op)
+static bool screen_opengl_render_anim_step(OGLRender *oglrender)
 {
-  OGLRender *oglrender = static_cast<OGLRender *>(op->customdata);
   Scene *scene = oglrender->scene;
   Depsgraph *depsgraph = oglrender->depsgraph;
   char filepath[FILE_MAX];
@@ -1143,21 +1134,27 @@ static bool screen_opengl_render_anim_step(bContext *C, wmOperator *op)
                                  nullptr);
 
     if ((scene->r.mode & R_NO_OVERWRITE) && BLI_exists(filepath)) {
-      BLI_spin_lock(&oglrender->reports_lock);
-      BKE_reportf(op->reports, RPT_INFO, "Skipping existing frame \"%s\"", filepath);
-      BLI_spin_unlock(&oglrender->reports_lock);
+      {
+        std::unique_lock lock(oglrender->reports_mutex);
+        BKE_reportf(oglrender->reports, RPT_INFO, "Skipping existing frame \"%s\"", filepath);
+      }
       ok = true;
       goto finally;
     }
   }
 
-  WM_cursor_time(oglrender->win, scene->r.cfra);
+  if (!oglrender->wm_job && oglrender->win) {
+    /* When doing blocking animation render without a job from a Python script, show time cursor so
+     * Blender doesn't appear frozen. */
+    WM_cursor_time(oglrender->win, scene->r.cfra);
+  }
 
   BKE_scene_graph_update_for_newframe(depsgraph);
 
   if (view_context) {
     if (oglrender->rv3d->persp == RV3D_CAMOB && oglrender->v3d->camera &&
-        oglrender->v3d->scenelock) {
+        oglrender->v3d->scenelock)
+    {
       /* since BKE_scene_graph_update_for_newframe() is used rather
        * then ED_update_for_newframe() the camera needs to be set */
       if (BKE_scene_camera_switch_update(scene)) {
@@ -1173,7 +1170,7 @@ static bool screen_opengl_render_anim_step(bContext *C, wmOperator *op)
       BLI_BITMAP_TEST_BOOL(oglrender->render_frames, scene->r.cfra - PSFRA))
   {
     /* render into offscreen buffer */
-    screen_opengl_render_apply(C, oglrender);
+    screen_opengl_render_apply(oglrender);
   }
 
   /* save to disk */
@@ -1192,7 +1189,6 @@ finally: /* Step the frame and bail early if needed */
 
   /* stop at the end or on error */
   if (scene->r.cfra >= PEFRA || !ok) {
-    screen_opengl_render_end(C, static_cast<OGLRender *>(op->customdata));
     return false;
   }
 
@@ -1202,48 +1198,70 @@ finally: /* Step the frame and bail early if needed */
 static int screen_opengl_render_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   OGLRender *oglrender = static_cast<OGLRender *>(op->customdata);
-  const bool anim = RNA_boolean_get(op->ptr, "animation");
-  bool ret;
 
-  switch (event->type) {
-    case EVT_ESCKEY:
-      /* cancel */
-      oglrender->pool_ok = false; /* Flag pool for cancel. */
-      screen_opengl_render_end(C, static_cast<OGLRender *>(op->customdata));
-      return OPERATOR_FINISHED;
-    case TIMER:
-      /* render frame? */
-      if (oglrender->timer == event->customdata) {
-        break;
-      }
-      ATTR_FALLTHROUGH;
-    default:
-      /* nothing to do */
-      return OPERATOR_RUNNING_MODAL;
-  }
-
-  /* run first because screen_opengl_render_anim_step can free oglrender */
-  WM_event_add_notifier(C, NC_SCENE | ND_RENDER_RESULT, oglrender->scene);
-
-  if (anim == 0) {
-    screen_opengl_render_apply(C, static_cast<OGLRender *>(op->customdata));
-    screen_opengl_render_end(C, static_cast<OGLRender *>(op->customdata));
+  /* Still render completes immediately, but still modal to show some feedback
+   * in case render initialization takes a while. */
+  if (!oglrender->is_animation) {
+    screen_opengl_render_apply(oglrender);
+    screen_opengl_render_end(oglrender);
+    MEM_delete(oglrender);
     return OPERATOR_FINISHED;
   }
 
-  ret = screen_opengl_render_anim_step(C, op);
-
-  /* stop at the end or on error */
-  if (ret == false) {
-    return OPERATOR_FINISHED;
+  /* no running blender, remove handler and pass through */
+  if (0 == WM_jobs_test(CTX_wm_manager(C), oglrender->scene, WM_JOB_TYPE_RENDER)) {
+    screen_opengl_render_end(oglrender);
+    MEM_delete(oglrender);
+    return OPERATOR_FINISHED | OPERATOR_PASS_THROUGH;
   }
 
-  return OPERATOR_RUNNING_MODAL;
+  /* catch escape key. */
+  return (event->type == EVT_ESCKEY) ? OPERATOR_RUNNING_MODAL : OPERATOR_PASS_THROUGH;
+}
+
+static void opengl_render_startjob(void *customdata, wmJobWorkerStatus *worker_status)
+{
+  OGLRender *oglrender = static_cast<OGLRender *>(customdata);
+  Scene *scene = oglrender->scene;
+
+  bool canceled = false;
+  bool finished = false;
+
+  while (!finished && !canceled) {
+    /* Render while blocking main thread, since we use 3D viewport resources. */
+    WM_job_main_thread_lock_acquire(oglrender->wm_job);
+
+    if (worker_status->stop || G.is_break) {
+      canceled = true;
+    }
+    else {
+      finished = !screen_opengl_render_anim_step(oglrender);
+      worker_status->progress = float(scene->r.cfra - PSFRA + 1) / float(PEFRA - PSFRA + 1);
+      worker_status->do_update = true;
+    }
+
+    WM_job_main_thread_lock_release(oglrender->wm_job);
+
+    if (worker_status->stop || G.is_break) {
+      canceled = true;
+    }
+  }
+
+  if (canceled) {
+    /* Cancel task pool writing images asynchronously. */
+    oglrender->pool_ok = false;
+  }
+}
+
+static void opengl_render_freejob(void *customdata)
+{
+  /* End the render here, as the modal handler might be called with the window out of focus. */
+  OGLRender *oglrender = static_cast<OGLRender *>(customdata);
+  screen_opengl_render_end(oglrender);
 }
 
 static int screen_opengl_render_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
-  OGLRender *oglrender;
   const bool anim = RNA_boolean_get(op->ptr, "animation");
 
   if (!screen_opengl_render_init(C, op)) {
@@ -1251,19 +1269,36 @@ static int screen_opengl_render_invoke(bContext *C, wmOperator *op, const wmEven
   }
 
   if (anim) {
-    if (!screen_opengl_render_anim_init(C, op)) {
+    if (!screen_opengl_render_anim_init(op)) {
       return OPERATOR_CANCELLED;
     }
   }
 
-  oglrender = static_cast<OGLRender *>(op->customdata);
+  OGLRender *oglrender = static_cast<OGLRender *>(op->customdata);
   render_view_open(C, event->xy[0], event->xy[1], op->reports);
 
   /* View may be changed above #USER_RENDER_DISPLAY_WINDOW. */
   oglrender->win = CTX_wm_window(C);
 
+  /* Setup animation job. */
+  if (anim) {
+    G.is_break = false;
+
+    wmJob *wm_job = WM_jobs_get(CTX_wm_manager(C),
+                                CTX_wm_window(C),
+                                oglrender->scene,
+                                "Viewport Render",
+                                WM_JOB_EXCL_RENDER | WM_JOB_PRIORITY | WM_JOB_PROGRESS,
+                                WM_JOB_TYPE_RENDER);
+    WM_jobs_customdata_set(wm_job, oglrender, opengl_render_freejob);
+    WM_jobs_timer(wm_job, 0.01f, NC_SCENE | ND_RENDER_RESULT, 0);
+    WM_jobs_callbacks(wm_job, opengl_render_startjob, nullptr, nullptr, nullptr);
+    WM_jobs_start(CTX_wm_manager(C), wm_job);
+
+    oglrender->wm_job = wm_job;
+  }
+
   WM_event_add_modal_handler(C, op);
-  oglrender->timer = WM_event_timer_add(oglrender->wm, oglrender->win, TIMER, 0.01f);
 
   return OPERATOR_RUNNING_MODAL;
 }
@@ -1271,40 +1306,39 @@ static int screen_opengl_render_invoke(bContext *C, wmOperator *op, const wmEven
 /* executes blocking render */
 static int screen_opengl_render_exec(bContext *C, wmOperator *op)
 {
-  const bool is_animation = RNA_boolean_get(op->ptr, "animation");
-
   if (!screen_opengl_render_init(C, op)) {
     return OPERATOR_CANCELLED;
   }
 
-  if (!is_animation) { /* same as invoke */
-    /* render image */
-    screen_opengl_render_apply(C, static_cast<OGLRender *>(op->customdata));
-    screen_opengl_render_end(C, static_cast<OGLRender *>(op->customdata));
+  OGLRender *oglrender = static_cast<OGLRender *>(op->customdata);
+
+  if (!oglrender->is_animation) { /* same as invoke */
+    screen_opengl_render_apply(oglrender);
+    screen_opengl_render_end(oglrender);
+    MEM_delete(oglrender);
 
     return OPERATOR_FINISHED;
   }
 
   bool ret = true;
 
-  if (!screen_opengl_render_anim_init(C, op)) {
+  if (!screen_opengl_render_anim_init(op)) {
     return OPERATOR_CANCELLED;
   }
 
   while (ret) {
-    ret = screen_opengl_render_anim_step(C, op);
+    ret = screen_opengl_render_anim_step(oglrender);
   }
 
-  /* no redraw needed, we leave state as we entered it */
-  //  ED_update_for_newframe(C);
-  WM_event_add_notifier(C, NC_SCENE | ND_RENDER_RESULT, CTX_data_scene(C));
+  screen_opengl_render_end(oglrender);
+  MEM_delete(oglrender);
 
   return OPERATOR_FINISHED;
 }
 
-static std::string screen_opengl_render_description(bContext * /*C*/,
-                                                    wmOperatorType * /*ot*/,
-                                                    PointerRNA *ptr)
+static std::string screen_opengl_render_get_description(bContext * /*C*/,
+                                                        wmOperatorType * /*ot*/,
+                                                        PointerRNA *ptr)
 {
   if (!RNA_boolean_get(ptr, "animation")) {
     return "";
@@ -1329,7 +1363,7 @@ void RENDER_OT_opengl(wmOperatorType *ot)
   ot->idname = "RENDER_OT_opengl";
 
   /* api callbacks */
-  ot->get_description = screen_opengl_render_description;
+  ot->get_description = screen_opengl_render_get_description;
   ot->invoke = screen_opengl_render_invoke;
   ot->exec = screen_opengl_render_exec; /* blocking */
   ot->modal = screen_opengl_render_modal;
@@ -1360,7 +1394,7 @@ void RENDER_OT_opengl(wmOperatorType *ot)
       "write_still",
       false,
       "Write Image",
-      "Save rendered the image to the output path (used only when animation is disabled)");
+      "Save the rendered image to the output path (used only when animation is disabled)");
   RNA_def_property_flag(prop, PROP_SKIP_SAVE);
   prop = RNA_def_boolean(ot->srna,
                          "view_context",
