@@ -17,13 +17,13 @@
 
 #include "BLI_listbase.h"
 #include "BLI_math_vector.h"
-#include "BLI_path_util.h"
+#include "BLI_path_utils.hh"
 #include "BLI_string.h"
 #include "BLI_utildefines.h"
 
-#include "BLT_translation.h"
+#include "BLT_translation.hh"
 
-#include "BKE_context.h"
+#include "BKE_context.hh"
 #include "BKE_movieclip.h"
 #include "BKE_screen.hh"
 #include "BKE_tracking.h"
@@ -41,10 +41,14 @@
 #include "WM_api.hh"
 #include "WM_types.hh"
 
-#include "IMB_imbuf.h"
-#include "IMB_imbuf_types.h"
+#include "IMB_imbuf.hh"
+#include "IMB_imbuf_types.hh"
 
-#include "clip_intern.h" /* own include */
+#include "MOV_read.hh"
+
+#include "clip_intern.hh" /* own include */
+
+using blender::StringRefNull;
 
 /* Panels */
 
@@ -85,17 +89,22 @@ void ED_clip_buttons_register(ARegionType *art)
 
 /********************* MovieClip Template ************************/
 
-void uiTemplateMovieClip(
-    uiLayout *layout, bContext *C, PointerRNA *ptr, const char *propname, bool compact)
+void uiTemplateMovieClip(uiLayout *layout,
+                         bContext *C,
+                         PointerRNA *ptr,
+                         const blender::StringRefNull propname,
+                         bool compact)
 {
   if (!ptr->data) {
     return;
   }
 
-  PropertyRNA *prop = RNA_struct_find_property(ptr, propname);
+  PropertyRNA *prop = RNA_struct_find_property(ptr, propname.c_str());
   if (!prop) {
-    printf(
-        "%s: property not found: %s.%s\n", __func__, RNA_struct_identifier(ptr->type), propname);
+    printf("%s: property not found: %s.%s\n",
+           __func__,
+           RNA_struct_identifier(ptr->type),
+           propname.c_str());
     return;
   }
 
@@ -103,7 +112,7 @@ void uiTemplateMovieClip(
     printf("%s: expected pointer property for %s.%s\n",
            __func__,
            RNA_struct_identifier(ptr->type),
-           propname);
+           propname.c_str());
     return;
   }
 
@@ -113,23 +122,13 @@ void uiTemplateMovieClip(
   uiLayoutSetContextPointer(layout, "edit_movieclip", &clipptr);
 
   if (!compact) {
-    uiTemplateID(layout,
-                 C,
-                 ptr,
-                 propname,
-                 nullptr,
-                 "CLIP_OT_open",
-                 nullptr,
-                 UI_TEMPLATE_ID_FILTER_ALL,
-                 false,
-                 nullptr);
+    uiTemplateID(layout, C, ptr, propname, nullptr, "CLIP_OT_open", nullptr);
   }
 
   if (clip) {
     uiLayout *row = uiLayoutRow(layout, false);
     uiBlock *block = uiLayoutGetBlock(row);
-    uiDefBut(
-        block, UI_BTYPE_LABEL, 0, IFACE_("File Path:"), 0, 19, 145, 19, nullptr, 0, 0, 0, 0, "");
+    uiDefBut(block, UI_BTYPE_LABEL, 0, IFACE_("File Path:"), 0, 19, 145, 19, nullptr, 0, 0, "");
 
     row = uiLayoutRow(layout, false);
     uiLayout *split = uiLayoutSplit(row, 0.0f, false);
@@ -145,16 +144,18 @@ void uiTemplateMovieClip(
 
 /********************* Track Template ************************/
 
-void uiTemplateTrack(uiLayout *layout, PointerRNA *ptr, const char *propname)
+void uiTemplateTrack(uiLayout *layout, PointerRNA *ptr, const StringRefNull propname)
 {
   if (!ptr->data) {
     return;
   }
 
-  PropertyRNA *prop = RNA_struct_find_property(ptr, propname);
+  PropertyRNA *prop = RNA_struct_find_property(ptr, propname.c_str());
   if (!prop) {
-    printf(
-        "%s: property not found: %s.%s\n", __func__, RNA_struct_identifier(ptr->type), propname);
+    printf("%s: property not found: %s.%s\n",
+           __func__,
+           RNA_struct_identifier(ptr->type),
+           propname.c_str());
     return;
   }
 
@@ -162,7 +163,7 @@ void uiTemplateTrack(uiLayout *layout, PointerRNA *ptr, const char *propname)
     printf("%s: expected pointer property for %s.%s\n",
            __func__,
            RNA_struct_identifier(ptr->type),
-           propname);
+           propname.c_str());
     return;
   }
 
@@ -190,8 +191,6 @@ void uiTemplateTrack(uiLayout *layout, PointerRNA *ptr, const char *propname)
            scopes,
            0,
            0,
-           0,
-           0,
            "");
 
   /* Resize grip. */
@@ -206,8 +205,6 @@ void uiTemplateTrack(uiLayout *layout, PointerRNA *ptr, const char *propname)
                 &scopes->track_preview_height,
                 UI_UNIT_Y,
                 UI_UNIT_Y * 20.0f,
-                0.0f,
-                0.0f,
                 "");
 }
 
@@ -378,7 +375,7 @@ static void marker_block_handler(bContext *C, void *arg_cb, int event)
 
 void uiTemplateMarker(uiLayout *layout,
                       PointerRNA *ptr,
-                      const char *propname,
+                      const StringRefNull propname,
                       PointerRNA *userptr,
                       PointerRNA *trackptr,
                       bool compact)
@@ -387,10 +384,12 @@ void uiTemplateMarker(uiLayout *layout,
     return;
   }
 
-  PropertyRNA *prop = RNA_struct_find_property(ptr, propname);
+  PropertyRNA *prop = RNA_struct_find_property(ptr, propname.c_str());
   if (!prop) {
-    printf(
-        "%s: property not found: %s.%s\n", __func__, RNA_struct_identifier(ptr->type), propname);
+    printf("%s: property not found: %s.%s\n",
+           __func__,
+           RNA_struct_identifier(ptr->type),
+           propname.c_str());
     return;
   }
 
@@ -398,7 +397,7 @@ void uiTemplateMarker(uiLayout *layout,
     printf("%s: expected pointer property for %s.%s\n",
            __func__,
            RNA_struct_identifier(ptr->type),
-           propname);
+           propname.c_str());
     return;
   }
 
@@ -442,8 +441,6 @@ void uiTemplateMarker(uiLayout *layout,
                                  &cb->marker_flag,
                                  0,
                                  0,
-                                 1,
-                                 0,
                                  tip);
     UI_but_funcN_set(bt, marker_update_cb, cb, nullptr);
     UI_but_drawflag_enable(bt, UI_BUT_ICON_REVERSE);
@@ -467,10 +464,8 @@ void uiTemplateMarker(uiLayout *layout,
                nullptr,
                0,
                0,
-               0,
-               0,
                "");
-
+      MEM_freeN(cb);
       return;
     }
 
@@ -520,8 +515,6 @@ void uiTemplateMarker(uiLayout *layout,
                  &cb->marker_flag,
                  0,
                  0,
-                 0,
-                 0,
                  tip);
 
     uiLayout *col = uiLayoutColumn(layout, true);
@@ -541,8 +534,6 @@ void uiTemplateMarker(uiLayout *layout,
              nullptr,
              0,
              0,
-             0,
-             0,
              "");
     uiBut *bt = uiDefButF(block,
                           UI_BTYPE_NUM,
@@ -555,8 +546,6 @@ void uiTemplateMarker(uiLayout *layout,
                           &cb->marker_pos[0],
                           -10 * width,
                           10.0 * width,
-                          0,
-                          0,
                           TIP_("X-position of marker at frame in screen coordinates"));
     UI_but_number_step_size_set(bt, step);
     UI_but_number_precision_set(bt, digits);
@@ -571,8 +560,6 @@ void uiTemplateMarker(uiLayout *layout,
                    &cb->marker_pos[1],
                    -10 * height,
                    10.0 * height,
-                   0,
-                   0,
                    TIP_("Y-position of marker at frame in screen coordinates"));
     UI_but_number_step_size_set(bt, step);
     UI_but_number_precision_set(bt, digits);
@@ -588,8 +575,6 @@ void uiTemplateMarker(uiLayout *layout,
              nullptr,
              0,
              0,
-             0,
-             0,
              "");
     bt = uiDefButF(block,
                    UI_BTYPE_NUM,
@@ -602,8 +587,6 @@ void uiTemplateMarker(uiLayout *layout,
                    &cb->track_offset[0],
                    -10 * width,
                    10.0 * width,
-                   0,
-                   0,
                    TIP_("X-offset to parenting point"));
     UI_but_number_step_size_set(bt, step);
     UI_but_number_precision_set(bt, digits);
@@ -618,8 +601,6 @@ void uiTemplateMarker(uiLayout *layout,
                    &cb->track_offset[1],
                    -10 * height,
                    10.0 * height,
-                   0,
-                   0,
                    TIP_("Y-offset to parenting point"));
     UI_but_number_step_size_set(bt, step);
     UI_but_number_precision_set(bt, digits);
@@ -635,8 +616,6 @@ void uiTemplateMarker(uiLayout *layout,
              nullptr,
              0,
              0,
-             0,
-             0,
              "");
     bt = uiDefButF(block,
                    UI_BTYPE_NUM,
@@ -649,8 +628,6 @@ void uiTemplateMarker(uiLayout *layout,
                    &cb->marker_pat[0],
                    3.0f,
                    10.0 * width,
-                   0,
-                   0,
                    TIP_("Width of marker's pattern in screen coordinates"));
     UI_but_number_step_size_set(bt, step);
     UI_but_number_precision_set(bt, digits);
@@ -665,8 +642,6 @@ void uiTemplateMarker(uiLayout *layout,
                    &cb->marker_pat[1],
                    3.0f,
                    10.0 * height,
-                   0,
-                   0,
                    TIP_("Height of marker's pattern in screen coordinates"));
     UI_but_number_step_size_set(bt, step);
     UI_but_number_precision_set(bt, digits);
@@ -682,8 +657,6 @@ void uiTemplateMarker(uiLayout *layout,
              nullptr,
              0,
              0,
-             0,
-             0,
              "");
     bt = uiDefButF(block,
                    UI_BTYPE_NUM,
@@ -696,8 +669,6 @@ void uiTemplateMarker(uiLayout *layout,
                    &cb->marker_search_pos[0],
                    -width,
                    width,
-                   0,
-                   0,
                    TIP_("X-position of search at frame relative to marker's position"));
     UI_but_number_step_size_set(bt, step);
     UI_but_number_precision_set(bt, digits);
@@ -712,8 +683,6 @@ void uiTemplateMarker(uiLayout *layout,
                    &cb->marker_search_pos[1],
                    -height,
                    height,
-                   0,
-                   0,
                    TIP_("Y-position of search at frame relative to marker's position"));
     UI_but_number_step_size_set(bt, step);
     UI_but_number_precision_set(bt, digits);
@@ -728,8 +697,6 @@ void uiTemplateMarker(uiLayout *layout,
                    &cb->marker_search[0],
                    3.0f,
                    10.0 * width,
-                   0,
-                   0,
                    TIP_("Width of marker's search in screen coordinates"));
     UI_but_number_step_size_set(bt, step);
     UI_but_number_precision_set(bt, digits);
@@ -744,8 +711,6 @@ void uiTemplateMarker(uiLayout *layout,
                    &cb->marker_search[1],
                    3.0f,
                    10.0 * height,
-                   0,
-                   0,
                    TIP_("Height of marker's search in screen coordinates"));
     UI_but_number_step_size_set(bt, step);
     UI_but_number_precision_set(bt, digits);
@@ -758,17 +723,19 @@ void uiTemplateMarker(uiLayout *layout,
 
 void uiTemplateMovieclipInformation(uiLayout *layout,
                                     PointerRNA *ptr,
-                                    const char *propname,
+                                    const StringRefNull propname,
                                     PointerRNA *userptr)
 {
   if (!ptr->data) {
     return;
   }
 
-  PropertyRNA *prop = RNA_struct_find_property(ptr, propname);
+  PropertyRNA *prop = RNA_struct_find_property(ptr, propname.c_str());
   if (!prop) {
-    printf(
-        "%s: property not found: %s.%s\n", __func__, RNA_struct_identifier(ptr->type), propname);
+    printf("%s: property not found: %s.%s\n",
+           __func__,
+           RNA_struct_identifier(ptr->type),
+           propname.c_str());
     return;
   }
 
@@ -776,7 +743,7 @@ void uiTemplateMovieclipInformation(uiLayout *layout,
     printf("%s: expected pointer property for %s.%s\n",
            __func__,
            RNA_struct_identifier(ptr->type),
-           propname);
+           propname.c_str());
     return;
   }
 
@@ -791,7 +758,7 @@ void uiTemplateMovieclipInformation(uiLayout *layout,
    * as metadata panel. So if the cache is skipped here it is not really a memory saver, but
    * skipping the cache could lead to a performance impact depending on the order in which panels
    * and the main area is drawn. Basically, if it is this template drawn first and then the main
-   * area it will lead to frame read and processing  happening twice. */
+   * area it will lead to frame read and processing happening twice. */
   ImBuf *ibuf = BKE_movieclip_get_ibuf_flag(clip, user, clip->flag, 0);
 
   int width, height;
@@ -800,52 +767,51 @@ void uiTemplateMovieclipInformation(uiLayout *layout,
 
   char str[1024];
   size_t ofs = 0;
-  ofs += BLI_snprintf_rlen(str + ofs, sizeof(str) - ofs, TIP_("%d x %d"), width, height);
+  ofs += BLI_snprintf_rlen(str + ofs, sizeof(str) - ofs, RPT_("%d x %d"), width, height);
 
   if (ibuf) {
     if (ibuf->float_buffer.data) {
       if (ibuf->channels != 4) {
         ofs += BLI_snprintf_rlen(
-            str + ofs, sizeof(str) - ofs, TIP_(", %d float channel(s)"), ibuf->channels);
+            str + ofs, sizeof(str) - ofs, RPT_(", %d float channel(s)"), ibuf->channels);
       }
       else if (ibuf->planes == R_IMF_PLANES_RGBA) {
-        ofs += BLI_strncpy_rlen(str + ofs, TIP_(", RGBA float"), sizeof(str) - ofs);
+        ofs += BLI_strncpy_rlen(str + ofs, RPT_(", RGBA float"), sizeof(str) - ofs);
       }
       else {
-        ofs += BLI_strncpy_rlen(str + ofs, TIP_(", RGB float"), sizeof(str) - ofs);
+        ofs += BLI_strncpy_rlen(str + ofs, RPT_(", RGB float"), sizeof(str) - ofs);
       }
     }
     else {
       if (ibuf->planes == R_IMF_PLANES_RGBA) {
-        ofs += BLI_strncpy_rlen(str + ofs, TIP_(", RGBA byte"), sizeof(str) - ofs);
+        ofs += BLI_strncpy_rlen(str + ofs, RPT_(", RGBA byte"), sizeof(str) - ofs);
       }
       else {
-        ofs += BLI_strncpy_rlen(str + ofs, TIP_(", RGB byte"), sizeof(str) - ofs);
+        ofs += BLI_strncpy_rlen(str + ofs, RPT_(", RGB byte"), sizeof(str) - ofs);
       }
     }
 
     if (clip->anim != nullptr) {
-      short frs_sec;
-      float frs_sec_base;
-      if (IMB_anim_get_fps(clip->anim, &frs_sec, &frs_sec_base, true)) {
-        ofs += BLI_snprintf_rlen(
-            str + ofs, sizeof(str) - ofs, TIP_(", %.2f fps"), float(frs_sec) / frs_sec_base);
+      float fps = MOV_get_fps(clip->anim);
+      if (fps > 0.0f) {
+        ofs += BLI_snprintf_rlen(str + ofs, sizeof(str) - ofs, RPT_(", %.2f fps"), fps);
       }
     }
   }
   else {
-    ofs += BLI_strncpy_rlen(str + ofs, TIP_(", failed to load"), sizeof(str) - ofs);
+    ofs += BLI_strncpy_rlen(str + ofs, RPT_(", failed to load"), sizeof(str) - ofs);
   }
+  UNUSED_VARS(ofs);
 
   uiItemL(col, str, ICON_NONE);
 
   /* Display current frame number. */
   int framenr = BKE_movieclip_remap_scene_to_clip_frame(clip, user->framenr);
   if (framenr <= clip->len) {
-    SNPRINTF(str, TIP_("Frame: %d / %d"), framenr, clip->len);
+    SNPRINTF(str, RPT_("Frame: %d / %d"), framenr, clip->len);
   }
   else {
-    SNPRINTF(str, TIP_("Frame: - / %d"), clip->len);
+    SNPRINTF(str, RPT_("Frame: - / %d"), clip->len);
   }
   uiItemL(col, str, ICON_NONE);
 
@@ -862,7 +828,7 @@ void uiTemplateMovieclipInformation(uiLayout *layout,
       file = "-";
     }
 
-    SNPRINTF(str, TIP_("File: %s"), file);
+    SNPRINTF(str, RPT_("File: %s"), file);
 
     uiItemL(col, str, ICON_NONE);
   }
