@@ -13,21 +13,20 @@
 #include "BLI_rect.h"
 #include "BLI_utildefines.h"
 
-#include "BKE_context.h"
-#include "BKE_image.h"
-#include "BKE_main.h"
+#include "BKE_context.hh"
+#include "BKE_image.hh"
+#include "BKE_node_legacy_types.hh"
 
 #include "ED_gizmo_library.hh"
 #include "ED_screen.hh"
 
-#include "IMB_imbuf_types.h"
+#include "IMB_imbuf_types.hh"
 
 #include "MEM_guardedalloc.h"
 
 #include "RNA_access.hh"
-#include "RNA_prototypes.h"
+#include "RNA_prototypes.hh"
 
-#include "WM_api.hh"
 #include "WM_types.hh"
 
 #include "node_intern.hh"
@@ -52,13 +51,16 @@ static void node_gizmo_calc_matrix_space(const SpaceNode *snode,
 static void node_gizmo_calc_matrix_space_with_image_dims(const SpaceNode *snode,
                                                          const ARegion *region,
                                                          const float2 &image_dims,
+                                                         const float2 &image_offset,
                                                          float matrix_space[4][4])
 {
   unit_m4(matrix_space);
   mul_v3_fl(matrix_space[0], snode->zoom * image_dims.x);
   mul_v3_fl(matrix_space[1], snode->zoom * image_dims.y);
-  matrix_space[3][0] = ((region->winx / 2) + snode->xof) - ((image_dims.x / 2.0f) * snode->zoom);
-  matrix_space[3][1] = ((region->winy / 2) + snode->yof) - ((image_dims.y / 2.0f) * snode->zoom);
+  matrix_space[3][0] = ((region->winx / 2) + snode->xof) -
+                       ((image_dims.x / 2.0f - image_offset.x) * snode->zoom);
+  matrix_space[3][1] = ((region->winy / 2) + snode->yof) -
+                       ((image_dims.y / 2.0f - image_offset.y) * snode->zoom);
 }
 
 /** \} */
@@ -101,9 +103,9 @@ static bool WIDGETGROUP_node_transform_poll(const bContext *C, wmGizmoGroupType
   }
 
   if (snode && snode->edittree && snode->edittree->type == NTREE_COMPOSIT) {
-    bNode *node = nodeGetActive(snode->edittree);
+    bNode *node = bke::node_get_active(snode->edittree);
 
-    if (node && ELEM(node->type, CMP_NODE_VIEWER, CMP_NODE_SPLITVIEWER)) {
+    if (node && ELEM(node->type_legacy, CMP_NODE_VIEWER)) {
       return true;
     }
   }
@@ -149,7 +151,7 @@ static void WIDGETGROUP_node_transform_refresh(const bContext *C, wmGizmoGroup *
     /* Need to set property here for undo. TODO: would prefer to do this in _init. */
     SpaceNode *snode = CTX_wm_space_node(C);
 #if 0
-    PointerRNA nodeptr = RNA_pointer_create(snode->id, &RNA_SpaceNodeEditor, snode);
+    PointerRNA nodeptr = RNA_pointer_create_discrete(snode->id, &RNA_SpaceNodeEditor, snode);
     WM_gizmo_target_property_def_rna(cage, "offset", &nodeptr, "backdrop_offset", -1);
     WM_gizmo_target_property_def_rna(cage, "scale", &nodeptr, "backdrop_zoom", -1);
 #endif
@@ -192,6 +194,7 @@ struct NodeCropWidgetGroup {
 
   struct {
     float2 dims;
+    float2 offset;
   } state;
 
   struct {
@@ -207,38 +210,37 @@ static void gizmo_node_crop_update(NodeCropWidgetGroup *crop_group)
       crop_group->update_data.context, &crop_group->update_data.ptr, crop_group->update_data.prop);
 }
 
-static void two_xy_to_rect(const NodeTwoXYs *nxy, rctf *rect, const float2 &dims, bool is_relative)
+static void two_xy_to_rect(
+    const NodeTwoXYs *nxy, const float2 &dims, const float2 offset, bool is_relative, rctf *r_rect)
 {
   if (is_relative) {
-    rect->xmin = nxy->fac_x1;
-    rect->xmax = nxy->fac_x2;
-    rect->ymin = nxy->fac_y1;
-    rect->ymax = nxy->fac_y2;
+    r_rect->xmin = nxy->fac_x1 + (offset.x / dims.x);
+    r_rect->xmax = nxy->fac_x2 + (offset.x / dims.x);
+    r_rect->ymin = nxy->fac_y2 + (offset.y / dims.y);
+    r_rect->ymax = nxy->fac_y1 + (offset.y / dims.y);
   }
   else {
-    rect->xmin = nxy->x1 / dims.x;
-    rect->xmax = nxy->x2 / dims.x;
-    rect->ymin = nxy->y1 / dims.y;
-    rect->ymax = nxy->y2 / dims.y;
+    r_rect->xmin = (nxy->x1 + offset.x) / dims.x;
+    r_rect->xmax = (nxy->x2 + offset.x) / dims.x;
+    r_rect->ymin = (nxy->y2 + offset.y) / dims.y;
+    r_rect->ymax = (nxy->y1 + offset.y) / dims.y;
   }
 }
 
-static void two_xy_from_rect(NodeTwoXYs *nxy,
-                             const rctf *rect,
-                             const float2 &dims,
-                             bool is_relative)
+static void two_xy_from_rect(
+    NodeTwoXYs *nxy, const rctf *rect, const float2 &dims, const float2 &offset, bool is_relative)
 {
   if (is_relative) {
-    nxy->fac_x1 = rect->xmin;
-    nxy->fac_x2 = rect->xmax;
-    nxy->fac_y1 = rect->ymin;
-    nxy->fac_y2 = rect->ymax;
+    nxy->fac_x1 = rect->xmin - (offset.x / dims.x);
+    nxy->fac_x2 = rect->xmax - (offset.x / dims.x);
+    nxy->fac_y2 = rect->ymin - (offset.y / dims.y);
+    nxy->fac_y1 = rect->ymax - (offset.y / dims.y);
   }
   else {
-    nxy->x1 = rect->xmin * dims.x;
-    nxy->x2 = rect->xmax * dims.x;
-    nxy->y1 = rect->ymin * dims.y;
-    nxy->y2 = rect->ymax * dims.y;
+    nxy->x1 = rect->xmin * dims.x - offset.x;
+    nxy->x2 = rect->xmax * dims.x - offset.x;
+    nxy->y2 = rect->ymin * dims.y - offset.y;
+    nxy->y1 = rect->ymax * dims.y - offset.y;
   }
 }
 
@@ -250,12 +252,13 @@ static void gizmo_node_crop_prop_matrix_get(const wmGizmo *gz,
   float(*matrix)[4] = (float(*)[4])value_p;
   BLI_assert(gz_prop->type->array_length == 16);
   NodeCropWidgetGroup *crop_group = (NodeCropWidgetGroup *)gz->parent_gzgroup->customdata;
-  const float *dims = crop_group->state.dims;
+  const float2 dims = crop_group->state.dims;
+  const float2 offset = crop_group->state.offset;
   const bNode *node = (const bNode *)gz_prop->custom_func.user_data;
   const NodeTwoXYs *nxy = (const NodeTwoXYs *)node->storage;
   bool is_relative = bool(node->custom2);
   rctf rct;
-  two_xy_to_rect(nxy, &rct, dims, is_relative);
+  two_xy_to_rect(nxy, dims, offset, is_relative, &rct);
   matrix[0][0] = fabsf(BLI_rctf_size_x(&rct));
   matrix[1][1] = fabsf(BLI_rctf_size_y(&rct));
   matrix[3][0] = (BLI_rctf_cent_x(&rct) - 0.5f) * dims[0];
@@ -269,29 +272,22 @@ static void gizmo_node_crop_prop_matrix_set(const wmGizmo *gz,
   const float(*matrix)[4] = (const float(*)[4])value_p;
   BLI_assert(gz_prop->type->array_length == 16);
   NodeCropWidgetGroup *crop_group = (NodeCropWidgetGroup *)gz->parent_gzgroup->customdata;
-  const float *dims = crop_group->state.dims;
+  const float2 dims = crop_group->state.dims;
+  const float2 offset = crop_group->state.offset;
   bNode *node = (bNode *)gz_prop->custom_func.user_data;
   NodeTwoXYs *nxy = (NodeTwoXYs *)node->storage;
   bool is_relative = bool(node->custom2);
   rctf rct;
-  two_xy_to_rect(nxy, &rct, dims, is_relative);
-  const bool nx = rct.xmin > rct.xmax;
-  const bool ny = rct.ymin > rct.ymax;
+  two_xy_to_rect(nxy, dims, offset, is_relative, &rct);
   BLI_rctf_resize(&rct, fabsf(matrix[0][0]), fabsf(matrix[1][1]));
-  BLI_rctf_recenter(&rct, (matrix[3][0] / dims[0]) + 0.5f, (matrix[3][1] / dims[1]) + 0.5f);
+  BLI_rctf_recenter(&rct, ((matrix[3][0]) / dims[0]) + 0.5f, ((matrix[3][1]) / dims[1]) + 0.5f);
   rctf rct_isect{};
-  rct_isect.xmin = 0;
-  rct_isect.xmax = 1;
-  rct_isect.ymin = 0;
-  rct_isect.ymax = 1;
+  rct_isect.xmin = offset.x / dims.x;
+  rct_isect.xmax = offset.x / dims.x + 1;
+  rct_isect.ymin = offset.y;
+  rct_isect.ymax = offset.y / dims.y + 1;
   BLI_rctf_isect(&rct_isect, &rct, &rct);
-  if (nx) {
-    std::swap(rct.xmin, rct.xmax);
-  }
-  if (ny) {
-    std::swap(rct.ymin, rct.ymax);
-  }
-  two_xy_from_rect(nxy, &rct, dims, is_relative);
+  two_xy_from_rect(nxy, &rct, dims, offset, is_relative);
   gizmo_node_crop_update(crop_group);
 }
 
@@ -304,9 +300,9 @@ static bool WIDGETGROUP_node_crop_poll(const bContext *C, wmGizmoGroupType * /*g
   }
 
   if (snode && snode->edittree && snode->edittree->type == NTREE_COMPOSIT) {
-    bNode *node = nodeGetActive(snode->edittree);
+    bNode *node = bke::node_get_active(snode->edittree);
 
-    if (node && ELEM(node->type, CMP_NODE_CROP)) {
+    if (node && ELEM(node->type_legacy, CMP_NODE_CROP)) {
       /* ignore 'use_crop_size', we can't usefully edit the crop in this case. */
       if ((node->custom1 & (1 << 0)) == 0) {
         return true;
@@ -327,6 +323,9 @@ static void WIDGETGROUP_node_crop_setup(const bContext * /*C*/, wmGizmoGroup *gz
                ED_GIZMO_CAGE_XFORM_FLAG_TRANSLATE | ED_GIZMO_CAGE_XFORM_FLAG_SCALE);
 
   gzgroup->customdata = crop_group;
+  gzgroup->customdata_free = [](void *customdata) {
+    MEM_delete(static_cast<NodeCropWidgetGroup *>(customdata));
+  };
 }
 
 static void WIDGETGROUP_node_crop_draw_prepare(const bContext *C, wmGizmoGroup *gzgroup)
@@ -352,15 +351,16 @@ static void WIDGETGROUP_node_crop_refresh(const bContext *C, wmGizmoGroup *gzgro
   if (ibuf) {
     crop_group->state.dims[0] = (ibuf->x > 0) ? ibuf->x : 64.0f;
     crop_group->state.dims[1] = (ibuf->y > 0) ? ibuf->y : 64.0f;
+    copy_v2_v2(crop_group->state.offset, ima->runtime.backdrop_offset);
 
     RNA_float_set_array(gz->ptr, "dimensions", crop_group->state.dims);
     WM_gizmo_set_flag(gz, WM_GIZMO_HIDDEN, false);
 
     SpaceNode *snode = CTX_wm_space_node(C);
-    bNode *node = nodeGetActive(snode->edittree);
+    bNode *node = bke::node_get_active(snode->edittree);
 
     crop_group->update_data.context = (bContext *)C;
-    crop_group->update_data.ptr = RNA_pointer_create(
+    crop_group->update_data.ptr = RNA_pointer_create_discrete(
         (ID *)snode->edittree, &RNA_CompositorNodeCrop, node);
     crop_group->update_data.prop = RNA_struct_find_property(&crop_group->update_data.ptr,
                                                             "relative");
@@ -404,6 +404,7 @@ struct NodeSunBeamsWidgetGroup {
 
   struct {
     float2 dims;
+    float2 offset;
   } state;
 };
 
@@ -416,9 +417,9 @@ static bool WIDGETGROUP_node_sbeam_poll(const bContext *C, wmGizmoGroupType * /*
   }
 
   if (snode && snode->edittree && snode->edittree->type == NTREE_COMPOSIT) {
-    bNode *node = nodeGetActive(snode->edittree);
+    bNode *node = bke::node_get_active(snode->edittree);
 
-    if (node && ELEM(node->type, CMP_NODE_SUNBEAMS)) {
+    if (node && ELEM(node->type_legacy, CMP_NODE_SUNBEAMS)) {
       return true;
     }
   }
@@ -450,7 +451,7 @@ static void WIDGETGROUP_node_sbeam_draw_prepare(const bContext *C, wmGizmoGroup
   SpaceNode *snode = CTX_wm_space_node(C);
 
   node_gizmo_calc_matrix_space_with_image_dims(
-      snode, region, sbeam_group->state.dims, gz->matrix_space);
+      snode, region, sbeam_group->state.dims, sbeam_group->state.offset, gz->matrix_space);
 }
 
 static void WIDGETGROUP_node_sbeam_refresh(const bContext *C, wmGizmoGroup *gzgroup)
@@ -466,12 +467,13 @@ static void WIDGETGROUP_node_sbeam_refresh(const bContext *C, wmGizmoGroup *gzgr
   if (ibuf) {
     sbeam_group->state.dims[0] = (ibuf->x > 0) ? ibuf->x : 64.0f;
     sbeam_group->state.dims[1] = (ibuf->y > 0) ? ibuf->y : 64.0f;
+    copy_v2_v2(sbeam_group->state.offset, ima->runtime.backdrop_offset);
 
     SpaceNode *snode = CTX_wm_space_node(C);
-    bNode *node = nodeGetActive(snode->edittree);
+    bNode *node = bke::node_get_active(snode->edittree);
 
     /* Need to set property here for undo. TODO: would prefer to do this in _init. */
-    PointerRNA nodeptr = RNA_pointer_create(
+    PointerRNA nodeptr = RNA_pointer_create_discrete(
         (ID *)snode->edittree, &RNA_CompositorNodeSunBeams, node);
     WM_gizmo_target_property_def_rna(gz, "offset", &nodeptr, "source", -1);
 
@@ -509,6 +511,7 @@ struct NodeCornerPinWidgetGroup {
 
   struct {
     float2 dims;
+    float2 offset;
   } state;
 };
 
@@ -521,9 +524,9 @@ static bool WIDGETGROUP_node_corner_pin_poll(const bContext *C, wmGizmoGroupType
   }
 
   if (snode && snode->edittree && snode->edittree->type == NTREE_COMPOSIT) {
-    bNode *node = nodeGetActive(snode->edittree);
+    bNode *node = bke::node_get_active(snode->edittree);
 
-    if (node && ELEM(node->type, CMP_NODE_CORNERPIN)) {
+    if (node && ELEM(node->type_legacy, CMP_NODE_CORNERPIN)) {
       return true;
     }
   }
@@ -543,7 +546,7 @@ static void WIDGETGROUP_node_corner_pin_setup(const bContext * /*C*/, wmGizmoGro
 
     RNA_enum_set(gz->ptr, "draw_style", ED_GIZMO_MOVE_STYLE_CROSS_2D);
 
-    gz->scale_basis = 0.01f / 75.0;
+    gz->scale_basis = 0.05f / 75.0;
   }
 
   gzgroup->customdata = cpin_group;
@@ -558,7 +561,7 @@ static void WIDGETGROUP_node_corner_pin_draw_prepare(const bContext *C, wmGizmoG
 
   float matrix_space[4][4];
   node_gizmo_calc_matrix_space_with_image_dims(
-      snode, region, cpin_group->state.dims, matrix_space);
+      snode, region, cpin_group->state.dims, cpin_group->state.offset, matrix_space);
 
   for (int i = 0; i < 4; i++) {
     wmGizmo *gz = cpin_group->gizmos[i];
@@ -578,9 +581,10 @@ static void WIDGETGROUP_node_corner_pin_refresh(const bContext *C, wmGizmoGroup
   if (ibuf) {
     cpin_group->state.dims[0] = (ibuf->x > 0) ? ibuf->x : 64.0f;
     cpin_group->state.dims[1] = (ibuf->y > 0) ? ibuf->y : 64.0f;
+    copy_v2_v2(cpin_group->state.offset, ima->runtime.backdrop_offset);
 
     SpaceNode *snode = CTX_wm_space_node(C);
-    bNode *node = nodeGetActive(snode->edittree);
+    bNode *node = bke::node_get_active(snode->edittree);
 
     /* need to set property here for undo. TODO: would prefer to do this in _init. */
     int i = 0;
@@ -588,7 +592,8 @@ static void WIDGETGROUP_node_corner_pin_refresh(const bContext *C, wmGizmoGroup
       if (sock->type == SOCK_VECTOR) {
         wmGizmo *gz = cpin_group->gizmos[i++];
 
-        PointerRNA sockptr = RNA_pointer_create((ID *)snode->edittree, &RNA_NodeSocket, sock);
+        PointerRNA sockptr = RNA_pointer_create_discrete(
+            (ID *)snode->edittree, &RNA_NodeSocket, sock);
         WM_gizmo_target_property_def_rna(gz, "offset", &sockptr, "default_value", -1);
 
         WM_gizmo_set_flag(gz, WM_GIZMO_DRAW_MODAL, true);
