@@ -6,28 +6,17 @@
  * \ingroup sequencer
  */
 
-#include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_windowmanager_types.h"
 
-#include "BKE_context.h"
-
-#include "BLI_blenlib.h"
-#include "BLI_utildefines.h"
+#include "BKE_context.hh"
 
 #include "ED_screen.hh"
 
 #include "UI_view2d.hh"
 
-#include "SEQ_channels.h"
-#include "SEQ_sequencer.h"
-#include "SEQ_time.h"
-
 #include "WM_api.hh"
 
-#include "RNA_define.hh"
-#include "RNA_enum_types.hh"
-
 /* Own include. */
 #include "sequencer_intern.hh"
 
@@ -38,7 +27,7 @@ static int sequencer_rename_channel_invoke(bContext *C, wmOperator * /*op*/, con
   channel_draw_context_init(C, CTX_wm_region(C), &context);
   float mouse_y = UI_view2d_region_to_view_y(context.timeline_region_v2d, event->mval[1]);
 
-  sseq->runtime.rename_channel_index = mouse_y;
+  sseq->runtime->rename_channel_index = mouse_y;
   WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER, CTX_data_scene(C));
   return OPERATOR_FINISHED;
 }
