@@ -5,8 +5,9 @@
 #include "BKE_attribute_math.hh"
 #include "BKE_curves.hh"
 #include "BKE_curves_utils.hh"
-#include "BKE_geometry_set.hh"
+#include "BKE_deform.hh"
 
+#include "BLI_array_utils.hh"
 #include "BLI_task.hh"
 
 #include "GEO_subdivide_curves.hh"
@@ -130,6 +131,22 @@ static void subdivide_attribute_catmull_rom(const OffsetIndices<int> src_points_
   });
 }
 
+static HandleType aligned_or_free_handle_type(const HandleType type)
+{
+  switch (type) {
+    case BEZIER_HANDLE_FREE:
+      return BEZIER_HANDLE_FREE;
+    case BEZIER_HANDLE_AUTO:
+      return BEZIER_HANDLE_ALIGN;
+    case BEZIER_HANDLE_VECTOR:
+      return BEZIER_HANDLE_FREE;
+    case BEZIER_HANDLE_ALIGN:
+      return BEZIER_HANDLE_ALIGN;
+  }
+  BLI_assert_unreachable();
+  return BEZIER_HANDLE_FREE;
+}
+
 static void subdivide_bezier_segment(const float3 &position_prev,
                                      const float3 &handle_prev,
                                      const float3 &handle_next,
@@ -137,33 +154,25 @@ static void subdivide_bezier_segment(const float3 &position_prev,
                                      const HandleType type_prev,
                                      const HandleType type_next,
                                      const IndexRange segment_points,
+                                     const int dst_next_segment_start,
                                      MutableSpan<float3> dst_positions,
                                      MutableSpan<float3> dst_handles_l,
                                      MutableSpan<float3> dst_handles_r,
                                      MutableSpan<int8_t> dst_types_l,
-                                     MutableSpan<int8_t> dst_types_r,
-                                     const bool is_last_cyclic_segment)
+                                     MutableSpan<int8_t> dst_types_r)
 {
-  auto fill_segment_handle_types = [&](const HandleType type) {
-    /* Also change the left handle of the control point following the segment's points. And don't
-     * change the left handle of the first point, since that is part of the previous segment. */
-    dst_types_l.slice_safe(segment_points.shift(1)).fill(type);
-    dst_types_r.slice(segment_points).fill(type);
-  };
-
   if (bke::curves::bezier::segment_is_vector(type_prev, type_next)) {
     linear_interpolation(position_prev, position_next, dst_positions.slice(segment_points));
-    fill_segment_handle_types(BEZIER_HANDLE_VECTOR);
+    /* All of the segment handles should be vector handles. */
+    dst_types_r[segment_points.first()] = BEZIER_HANDLE_VECTOR;
+    dst_types_l[dst_next_segment_start] = BEZIER_HANDLE_VECTOR;
+    dst_types_l.slice(segment_points.drop_front(1)).fill(BEZIER_HANDLE_VECTOR);
+    dst_types_r.slice(segment_points.drop_front(1)).fill(BEZIER_HANDLE_VECTOR);
   }
   else {
     /* The first point in the segment is always copied. */
     dst_positions[segment_points.first()] = position_prev;
 
-    /* Non-vector segments in the result curve are given free handles. This could possibly be
-     * improved with another pass that sets handles to aligned where possible, but currently that
-     * does not provide much benefit for the increased complexity. */
-    fill_segment_handle_types(BEZIER_HANDLE_FREE);
-
     /* In order to generate a Bezier curve with the same shape as the input curve, apply the
      * De Casteljau algorithm iteratively for the provided number of cuts, constantly updating the
      * previous result point's right handle and the left handle at the end of the segment. */
@@ -190,9 +199,16 @@ static void subdivide_bezier_segment(const float3 &position_prev,
     }
 
     /* Copy the handles for the last segment from the working variables. */
-    const int i_segment_last = is_last_cyclic_segment ? 0 : segment_points.one_after_last();
     dst_handles_r[segment_points.last()] = segment_handle_prev;
-    dst_handles_l[i_segment_last] = segment_handle_next;
+    dst_handles_l[dst_next_segment_start] = segment_handle_next;
+
+    /* First and last handles at the ends of the segment are aligned if possible. */
+    dst_types_r[segment_points.first()] = aligned_or_free_handle_type(type_prev);
+    dst_types_l[dst_next_segment_start] = aligned_or_free_handle_type(type_next);
+
+    /* Handles inside the segment are aligned. */
+    dst_types_l.slice(segment_points.drop_front(1)).fill(BEZIER_HANDLE_ALIGN);
+    dst_types_r.slice(segment_points.drop_front(1)).fill(BEZIER_HANDLE_ALIGN);
   }
 }
 
@@ -219,42 +235,31 @@ static void subdivide_bezier_positions(const Span<float3> src_positions,
                                HandleType(src_types_r[segment_i]),
                                HandleType(src_types_l[segment_i + 1]),
                                segment,
+                               segment.one_after_last(),
                                dst_positions,
                                dst_handles_l,
                                dst_handles_r,
                                dst_types_l,
-                               dst_types_r,
-                               false);
+                               dst_types_r);
     }
   });
 
   if (cyclic) {
     const int last_index = src_positions.index_range().last();
     const IndexRange segment = evaluated_offsets[last_index];
-    const HandleType type_prev = HandleType(src_types_r.last());
-    const HandleType type_next = HandleType(src_types_l.first());
     subdivide_bezier_segment(src_positions.last(),
                              src_handles_r.last(),
                              src_handles_l.first(),
                              src_positions.first(),
-                             type_prev,
-                             type_next,
+                             HandleType(src_types_r.last()),
+                             HandleType(src_types_l.first()),
                              segment,
+                             0,
                              dst_positions,
                              dst_handles_l,
                              dst_handles_r,
                              dst_types_l,
-                             dst_types_r,
-                             true);
-
-    if (bke::curves::bezier::segment_is_vector(type_prev, type_next)) {
-      dst_types_l.first() = BEZIER_HANDLE_VECTOR;
-      dst_types_r.last() = BEZIER_HANDLE_VECTOR;
-    }
-    else {
-      dst_types_l.first() = BEZIER_HANDLE_FREE;
-      dst_types_r.last() = BEZIER_HANDLE_FREE;
-    }
+                             dst_types_r);
   }
   else {
     dst_positions.last() = src_positions.last();
@@ -269,13 +274,12 @@ static void subdivide_bezier_positions(const Span<float3> src_positions,
       cyclic, dst_types_l, dst_types_r, dst_positions, dst_handles_l, dst_handles_r);
 }
 
-bke::CurvesGeometry subdivide_curves(
-    const bke::CurvesGeometry &src_curves,
-    const IndexMask &selection,
-    const VArray<int> &cuts,
-    const bke::AnonymousAttributePropagationInfo &propagation_info)
+bke::CurvesGeometry subdivide_curves(const bke::CurvesGeometry &src_curves,
+                                     const IndexMask &selection,
+                                     const VArray<int> &cuts,
+                                     const bke::AttributeFilter &attribute_filter)
 {
-  if (src_curves.points_num() == 0) {
+  if (src_curves.is_empty()) {
     return src_curves;
   }
 
@@ -286,6 +290,8 @@ bke::CurvesGeometry subdivide_curves(
   const IndexMask unselected = selection.complement(src_curves.curves_range(), memory);
 
   bke::CurvesGeometry dst_curves = bke::curves::copy_only_curve_domain(src_curves);
+  /* Copy vertex groups from source curves to allow copying vertex group attributes. */
+  BKE_defgroup_copy_list(&dst_curves.vertex_group_names, &src_curves.vertex_group_names);
 
   /* For each point, this contains the point offset in the corresponding result curve,
    * starting at zero. For example for two curves with four points each, the values might
@@ -300,7 +306,7 @@ bke::CurvesGeometry subdivide_curves(
    * Storing the leading zero is unnecessary but makes the array a bit simpler to use by avoiding
    * a check for the first segment, and because some existing utilities also use leading zeros. */
   Array<int> all_point_offset_data(src_curves.points_num() + src_curves.curves_num());
-#ifdef DEBUG
+#ifndef NDEBUG
   all_point_offset_data.fill(-1);
 #endif
   calculate_result_offsets(src_curves,
@@ -319,10 +325,12 @@ bke::CurvesGeometry subdivide_curves(
   const bke::AttributeAccessor src_attributes = src_curves.attributes();
   bke::MutableAttributeAccessor dst_attributes = dst_curves.attributes_for_write();
 
+  Vector<bke::AttributeTransferData> attributes_to_transfer =
+      bke::retrieve_attributes_for_transfer(
+          src_attributes, dst_attributes, ATTR_DOMAIN_MASK_POINT, attribute_filter);
+
   auto subdivide_catmull_rom = [&](const IndexMask &selection) {
-    for (auto &attribute : bke::retrieve_attributes_for_transfer(
-             src_attributes, dst_attributes, ATTR_DOMAIN_MASK_POINT, propagation_info))
-    {
+    for (auto &attribute : attributes_to_transfer) {
       subdivide_attribute_catmull_rom(src_points_by_curve,
                                       dst_points_by_curve,
                                       selection,
@@ -330,21 +338,17 @@ bke::CurvesGeometry subdivide_curves(
                                       cyclic,
                                       attribute.src,
                                       attribute.dst.span);
-      attribute.dst.finish();
     }
   };
 
   auto subdivide_poly = [&](const IndexMask &selection) {
-    for (auto &attribute : bke::retrieve_attributes_for_transfer(
-             src_attributes, dst_attributes, ATTR_DOMAIN_MASK_POINT, propagation_info))
-    {
+    for (auto &attribute : attributes_to_transfer) {
       subdivide_attribute_linear(src_points_by_curve,
                                  dst_points_by_curve,
                                  selection,
                                  all_point_offsets,
                                  attribute.src,
                                  attribute.dst.span);
-      attribute.dst.finish();
     }
   };
 
@@ -381,20 +385,19 @@ bke::CurvesGeometry subdivide_curves(
                                  dst_handles_r.slice(dst_points));
     });
 
-    for (auto &attribute : bke::retrieve_attributes_for_transfer(
-             src_attributes,
-             dst_attributes,
-             ATTR_DOMAIN_MASK_POINT,
-             propagation_info,
-             {"position", "handle_type_left", "handle_type_right", "handle_right", "handle_left"}))
-    {
+    /* Filter out positions and handles that are already interpolated. */
+    const Set<StringRef> attributes_to_skip = {
+        "position", "handle_type_left", "handle_type_right", "handle_right", "handle_left"};
+    for (auto &attribute : attributes_to_transfer) {
+      if (attributes_to_skip.contains(attribute.name)) {
+        continue;
+      }
       subdivide_attribute_linear(src_points_by_curve,
                                  dst_points_by_curve,
                                  selection,
                                  all_point_offsets,
                                  attribute.src,
                                  attribute.dst.span);
-      attribute.dst.finish();
     }
   };
 
@@ -410,14 +413,10 @@ bke::CurvesGeometry subdivide_curves(
                                      subdivide_bezier,
                                      subdivide_nurbs);
 
-  if (!unselected.is_empty()) {
-    for (auto &attribute : bke::retrieve_attributes_for_transfer(
-             src_attributes, dst_attributes, ATTR_DOMAIN_MASK_POINT, propagation_info))
-    {
-      bke::curves::copy_point_data(
-          src_points_by_curve, dst_points_by_curve, unselected, attribute.src, attribute.dst.span);
-      attribute.dst.finish();
-    }
+  for (auto &attribute : attributes_to_transfer) {
+    array_utils::copy_group_to_group(
+        src_points_by_curve, dst_points_by_curve, unselected, attribute.src, attribute.dst.span);
+    attribute.dst.finish();
   }
 
   return dst_curves;
