@@ -11,31 +11,32 @@
 #include <cstring>
 
 #include "BLI_math_vector.h"
+#include "BLI_math_vector_types.hh"
 #include "BLI_rect.h"
 
-#include "BKE_colortools.h"
+#include "BKE_colortools.hh"
 
-#include "IMB_colormanagement.h"
+#include "IMB_colormanagement.hh"
 
 #include "DNA_vec_types.h"
 
-#include "GPU_capabilities.h"
-#include "GPU_framebuffer.h"
-#include "GPU_immediate.h"
-#include "GPU_matrix.h"
-#include "GPU_texture.h"
-#include "GPU_uniform_buffer.h"
-#include "GPU_viewport.h"
+#include "GPU_capabilities.hh"
+#include "GPU_framebuffer.hh"
+#include "GPU_immediate.hh"
+#include "GPU_matrix.hh"
+#include "GPU_state.hh"
+#include "GPU_texture.hh"
+#include "GPU_viewport.hh"
 
-#include "DRW_engine.h"
+#include "DRW_engine.hh"
 
 #include "MEM_guardedalloc.h"
 
-/* Struct storing a viewport specific GPUBatch.
+/* Struct storing a viewport specific blender::gpu::Batch.
  * The end-goal is to have a single batch shared across viewport and use a model matrix to place
  * the batch. Due to OCIO and Image/UV editor we are not able to use an model matrix yet. */
 struct GPUViewportBatch {
-  GPUBatch *batch;
+  blender::gpu::Batch *batch;
   struct {
     rctf rect_pos;
     rctf rect_uv;
@@ -50,7 +51,7 @@ static struct {
 } g_viewport = {{0}};
 
 struct GPUViewport {
-  int size[2];
+  blender::int2 size;
   int flag;
 
   /* Set the active view (for stereoscopic viewport rendering). */
@@ -121,12 +122,12 @@ DRWData **GPU_viewport_data_get(GPUViewport *viewport)
 static void gpu_viewport_textures_create(GPUViewport *viewport)
 {
   int *size = viewport->size;
-  float empty_pixel[4] = {0.0f, 0.0f, 0.0f, 0.0f};
+  float const empty_pixel[4] = {0.0f, 0.0f, 0.0f, 0.0f};
   eGPUTextureUsage usage = GPU_TEXTURE_USAGE_SHADER_READ | GPU_TEXTURE_USAGE_ATTACHMENT;
 
   if (viewport->color_render_tx[0] == nullptr) {
 
-    /* NOTE: dtxl_color texture requires write support as it may be written to by the realtime
+    /* NOTE: dtxl_color texture requires write support as it may be written to by the viewport
      * compositor. */
     viewport->color_render_tx[0] = GPU_texture_create_2d("dtxl_color",
                                                          UNPACK2(size),
@@ -137,10 +138,8 @@ static void gpu_viewport_textures_create(GPUViewport *viewport)
     viewport->color_overlay_tx[0] = GPU_texture_create_2d(
         "dtxl_color_overlay", UNPACK2(size), 1, GPU_SRGB8_A8, usage, nullptr);
 
-    if (GPU_clear_viewport_workaround()) {
-      GPU_texture_clear(viewport->color_render_tx[0], GPU_DATA_FLOAT, empty_pixel);
-      GPU_texture_clear(viewport->color_overlay_tx[0], GPU_DATA_FLOAT, empty_pixel);
-    }
+    GPU_texture_clear(viewport->color_render_tx[0], GPU_DATA_FLOAT, empty_pixel);
+    GPU_texture_clear(viewport->color_overlay_tx[0], GPU_DATA_FLOAT, empty_pixel);
   }
 
   if ((viewport->flag & GPU_VIEWPORT_STEREO) != 0 && viewport->color_render_tx[1] == nullptr) {
@@ -153,10 +152,8 @@ static void gpu_viewport_textures_create(GPUViewport *viewport)
     viewport->color_overlay_tx[1] = GPU_texture_create_2d(
         "dtxl_color_overlay_stereo", UNPACK2(size), 1, GPU_SRGB8_A8, usage, nullptr);
 
-    if (GPU_clear_viewport_workaround()) {
-      GPU_texture_clear(viewport->color_render_tx[1], GPU_DATA_FLOAT, empty_pixel);
-      GPU_texture_clear(viewport->color_overlay_tx[1], GPU_DATA_FLOAT, empty_pixel);
-    }
+    GPU_texture_clear(viewport->color_render_tx[1], GPU_DATA_FLOAT, empty_pixel);
+    GPU_texture_clear(viewport->color_overlay_tx[1], GPU_DATA_FLOAT, empty_pixel);
   }
 
   /* Can be shared with #GPUOffscreen. */
@@ -168,12 +165,10 @@ static void gpu_viewport_textures_create(GPUViewport *viewport)
                                                1,
                                                GPU_DEPTH24_STENCIL8,
                                                usage | GPU_TEXTURE_USAGE_HOST_READ |
-                                                   GPU_TEXTURE_USAGE_MIP_SWIZZLE_VIEW,
+                                                   GPU_TEXTURE_USAGE_FORMAT_VIEW,
                                                nullptr);
-    if (GPU_clear_viewport_workaround()) {
-      static int depth_clear = 0;
-      GPU_texture_clear(viewport->depth_tx, GPU_DATA_UINT_24_8, &depth_clear);
-    }
+    const int depth_clear = 0;
+    GPU_texture_clear(viewport->depth_tx, GPU_DATA_UINT_24_8, &depth_clear);
   }
 
   if (!viewport->depth_tx || !viewport->color_render_tx[0] || !viewport->color_overlay_tx[0]) {
@@ -196,14 +191,14 @@ static void gpu_viewport_textures_free(GPUViewport *viewport)
 
 void GPU_viewport_bind(GPUViewport *viewport, int view, const rcti *rect)
 {
-  int rect_size[2];
+  blender::int2 rect_size;
   /* add one pixel because of scissor test */
   rect_size[0] = BLI_rcti_size_x(rect) + 1;
   rect_size[1] = BLI_rcti_size_y(rect) + 1;
 
   DRW_gpu_context_enable();
 
-  if (!equals_v2v2_int(viewport->size, rect_size)) {
+  if (viewport->size != rect_size) {
     copy_v2_v2_int(viewport->size, rect_size);
     gpu_viewport_textures_free(viewport);
     gpu_viewport_textures_create(viewport);
@@ -357,7 +352,7 @@ void GPU_viewport_stereo_composite(GPUViewport *viewport, Stereo3dFormat *stereo
 /** \name Viewport Batches
  * \{ */
 
-static GPUVertFormat *gpu_viewport_batch_format()
+static const GPUVertFormat &gpu_viewport_batch_format()
 {
   if (g_viewport.format.attr_len == 0) {
     GPUVertFormat *format = &g_viewport.format;
@@ -366,14 +361,14 @@ static GPUVertFormat *gpu_viewport_batch_format()
     g_viewport.attr_id.tex_coord = GPU_vertformat_attr_add(
         format, "texCoord", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
   }
-  return &g_viewport.format;
+  return g_viewport.format;
 }
 
-static GPUBatch *gpu_viewport_batch_create(const rctf *rect_pos, const rctf *rect_uv)
+static blender::gpu::Batch *gpu_viewport_batch_create(const rctf *rect_pos, const rctf *rect_uv)
 {
-  GPUVertBuf *vbo = GPU_vertbuf_create_with_format(gpu_viewport_batch_format());
+  blender::gpu::VertBuf *vbo = GPU_vertbuf_create_with_format(gpu_viewport_batch_format());
   const uint vbo_len = 4;
-  GPU_vertbuf_data_alloc(vbo, vbo_len);
+  GPU_vertbuf_data_alloc(*vbo, vbo_len);
 
   GPUVertBufRaw pos_step, tex_coord_step;
   GPU_vertbuf_attr_get_raw_data(vbo, g_viewport.attr_id.pos, &pos_step);
@@ -399,9 +394,9 @@ static GPUBatch *gpu_viewport_batch_create(const rctf *rect_pos, const rctf *rec
   return GPU_batch_create_ex(GPU_PRIM_TRI_STRIP, vbo, nullptr, GPU_BATCH_OWNS_VBO);
 }
 
-static GPUBatch *gpu_viewport_batch_get(GPUViewport *viewport,
-                                        const rctf *rect_pos,
-                                        const rctf *rect_uv)
+static blender::gpu::Batch *gpu_viewport_batch_get(GPUViewport *viewport,
+                                                   const rctf *rect_pos,
+                                                   const rctf *rect_uv)
 {
   const float compare_limit = 0.0001f;
   const bool parameters_changed =
@@ -463,7 +458,7 @@ static void gpu_viewport_draw_colormanaged(GPUViewport *viewport,
                                                               do_overlay_merge);
   }
 
-  GPUBatch *batch = gpu_viewport_batch_get(viewport, rect_pos, rect_uv);
+  blender::gpu::Batch *batch = gpu_viewport_batch_get(viewport, rect_pos, rect_uv);
   if (use_ocio) {
     GPU_batch_program_set_imm_shader(batch);
   }
@@ -526,10 +521,10 @@ void GPU_viewport_draw_to_screen_ex(GPUViewport *viewport,
   /* Mirror the UV rect in case axis-swapped drawing is requested (by passing a rect with min and
    * max values swapped). */
   if (BLI_rcti_size_x(rect) < 0) {
-    SWAP(float, uv_rect.xmin, uv_rect.xmax);
+    std::swap(uv_rect.xmin, uv_rect.xmax);
   }
   if (BLI_rcti_size_y(rect) < 0) {
-    SWAP(float, uv_rect.ymin, uv_rect.ymax);
+    std::swap(uv_rect.ymin, uv_rect.ymax);
   }
 
   gpu_viewport_draw_colormanaged(
